/* MAP's LINES - Library of rutines for work with line's marks in a map
 * Copyright (C) 1999-2001  Jan Lana
 *
 * $Id: ml.h,v 1.18 2001/04/19 15:31:14 mapa Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _M_ML_H
#define _M_ML_H

#include <glib.h>

#define ML_FSHIFT 11
#define ML_FIXED_1 (1<<ML_FSHIFT)

#define ML_LAYER_LtYL	0x0001
#define ML_LAYER_DkYL	0x0002
#define ML_LAYER_LtGR	0x0004
#define ML_LAYER_MdGR	0x0008
#define ML_LAYER_DkGR	0x0010
#define ML_LAYER_LtBL	0x0020
#define ML_LAYER_DkBL	0x0040
#define ML_LAYER_LtBR	0x0080
#define ML_LAYER_DkBR	0x0100
#define ML_LAYER_BK	0x0200
#define ML_LAYER_ERR	0x0400

/* every quality limits must by less that this constant */
#define ML_CONNECT_QUALITY_MAX		(10*ML_FIXED_1)

#define ABS_DIFF(x,y) ((x<y)?((y)-(x)):((x)-(y)))


typedef guint 		MLCoord;	/* coordinate = x * #cols + y */

typedef guint16 	MLLineIndex;	/* type of index to lines's table */
typedef gint32		MLFFloat;	/* fixed float */
typedef guint16		MLSeparations;

typedef struct _MLPictureSep 	MLPictureSep;
typedef struct _MLPictureUI8 	MLPictureUI8;
typedef struct _MLLineAtom	MLLineAtom;
typedef struct _MLAtoms		MLAtoms;
typedef struct _MLLine		MLLine;
typedef struct _MLLines		MLLines;
typedef struct _MLGlue		MLGlue;
typedef struct _MLStep		MLStep;
typedef struct _MLGlueParam	MLGlueParam;
typedef struct _MLConnectParam	MLConnectParam;

/* lineAtom type */
typedef enum {
  MLTA_UNKNOWN = 0,
  MLTA_LINE,		/* the atom is a part of a line-mark */
  MLTA_RASTER,		/* the atom is a part of a raster-mark */
  MLTA_MARK,		/* the atom is a part of a not-line-mark */
  MLTA_NOTLINE		/* the atom is an error. It's not a part of a line */
} MLTAtom;

/* glue type */
typedef enum {
  MLTG_UNKNOWN = 0,
  MLTG_SPACE,		/* the glue is a step throught space */
  MLTG_OVERPRINT,	/* the glue try to correct an overprint-error */
  MLTG_CROSS		/* the glue goes over a cross */
} MLTGlue;

/* line type */
typedef enum {
  MLTL_UNKNOWN = 0,
  MLTL_LONG,
  MLTL_DASHED,
  MLTL_DOTTED
} MLTLine;

typedef MLFFloat	(*MLGlueFuncStep) (const MLStep *curr,
					 MLSeparations	pix,
        				 guint8		step_direct, 
					 MLFFloat	new_direct,
					 MLFFloat	line_direct,
					 gboolean	is_cross);
typedef MLFFloat	(*MLGlueFuncCon) (const MLStep *curr,
					 MLSeparations	pix,
        				 guint8		step_direct, 
					 MLFFloat	new_direct,
					 MLFFloat	line_direct,
					 gboolean	is_cross,
					 const MLLineAtom *me,
					 guint8		me_tip,
					 const MLLineAtom *neighbour,
					 guint8		neighbour_tip);

typedef gboolean	(*MLConnectFuncSeed) (
					 const MLLine	*atom);
typedef gboolean	(*MLConnectFuncPossible) (
					 const MLLine	*line,
					 const MLLine	*line2,
					 const MLLineAtom *gl,
					 MLCoord	 line_tip,
					 MLCoord	 line2_tip);
typedef MLLineAtom*	(*MLConnectFuncPick) (
					 const MLLine	*line,
					 GPtrArray	*gls,
					 const MLLineAtom *gl2);

struct _MLPictureSep {
  MLCoord rows;
  MLCoord cols;
  MLSeparations *p;
};


struct _MLPictureUI8 {
  MLCoord rows;
  MLCoord cols;
  guint8 *p;
};


struct _MLLineAtom {
  guint len;
  MLCoord tip[2];	/* 0 - start-point; 1 - end-point */
  MLFFloat penalty;
  MLFFloat width;	/* avg of atom's width */
  MLFFloat diff_width;	/* sum(|width-width[i]|) / len */
  MLLine *line;
  guint8 *data;
  union {
    MLTAtom line;
    MLTGlue glue;
  } type;
};


struct _MLAtoms {
  GPtrArray *atoms;
  GHashTable *tips;	/* key - end (coord) of a line
			   value - pointer to 'atoms' */
};


struct _MLLine {
  guint len;		/* sum of atoms's len */
  MLCoord tip[2];	/* end points of line */
  MLTLine type;

  guint cycle : 1;
  MLFFloat penalty;	/* sum of connects's penalty */
  MLFFloat width;
  
  /* dashed stats */
  MLFFloat avg_black;
  MLFFloat avg_space;
  guint pieces;		/* start number is 0 ! */
  
  
  /* friendship */
  MLLineAtom *connect[2];	/* 0 - start-point; 1 - end-point */

  /* data */
  GPtrArray *atoms;	/* array of lineAtoms. 
			   !! I'm not OWNER of the lineAtoms !! */
};


struct _MLLines {
  GPtrArray *lines;
  GHashTable *tips;	/* key - end (coord) of a line,
			   value - pointer to the lineAtom */
};


struct _MLGlue {
  GPtrArray *atoms;
  GHashTable *tips;	/* key - end (coord) of a line
			   value - GPtrArray of atoms from the coord 
			           sorted by penalty (best - first) */
};


struct _MLStep {
  MLCoord coord;
  MLFFloat direct;
  MLFFloat penalty;
  guint8 direct_in;
  guint  len;
  struct _MLStep *in;
};


struct _MLGlueParam {
  MLGlueFuncStep step_penalty;
  MLGlueFuncCon  connect_penalty;
  MLFFloat max_penalty;
  MLTGlue glue_type;
  MLPictureUI8 *debug;
};


struct _MLConnectParam {
  MLConnectFuncSeed is_seed;
  MLConnectFuncPossible is_possible;
  MLConnectFuncPossible is_possible_cycle;
  MLConnectFuncPick pick;
  MLTLine type;
};



/* DIRECTB calcs with 0-7 (8-unknown), DIRECTF calcs with the
 * same numbers as fixed-point float (*= M_FIXED_1)
 */
#define ML_DIRECTB_UNKNOWN 8
#define ML_DIRECTF_UNKNOWN (ML_FIXED_1*ML_DIRECTB_UNKNOWN)

#define ML_DIRECT_RTURN(k,a) (((a)>=k*2)?((a)-k*2):((a)+k*6))
#define ML_DIRECT_LTURN(k,a) (((a)<k*6)?((a)+k*2):((a)-k*6))
#define ML_DIRECT_UTURN(k,a) (((a)>=k*4)?((a)-k*4):((a)+k*4))


/* 'r' - turn right ( 90 grad)
 * 'l' - turn left  (-90 grad)
 * 'u' - u-turn	    (180 grad)
 */
#define ml_directb_rturn(a) ML_DIRECT_RTURN(1,a)
#define ml_directb_lturn(a) ML_DIRECT_LTURN(1,a)
#define ml_directb_uturn(a) ML_DIRECT_UTURN(1,a)

#define ml_directf_rturn(a) ML_DIRECT_RTURN(ML_FIXED_1,a)
#define ml_directf_lturn(a) ML_DIRECT_LTURN(ML_FIXED_1,a)
#define ml_directf_uturn(a) ML_DIRECT_UTURN(ML_FIXED_1,a)

G_INLINE_FUNC MLFFloat ml_directf_calc (MLFFloat d, MLFFloat n, MLFFloat exp);
G_INLINE_FUNC MLFFloat ml_directb_diff (MLFFloat a, MLFFloat b);
G_INLINE_FUNC MLFFloat ml_directf_diff (MLFFloat a, MLFFloat b);


#ifdef G_CAN_INLINE
G_INLINE_FUNC MLFFloat ml_directf_calc (MLFFloat d, MLFFloat n, MLFFloat exp)
{
  g_assert (n != ML_DIRECTF_UNKNOWN);

  if (d == ML_DIRECTF_UNKNOWN)
    return n;

  if (n<d) {
    if (d-n > ML_FIXED_1*4)
      n += ML_FIXED_1*8;
  } else
    if (n-d > ML_FIXED_1*4)
      n -= ML_FIXED_1*8;

  d *= exp;
  d += n*(ML_FIXED_1-exp);
  if (d < 0)
    d += ML_FIXED_1*ML_FIXED_1*8;
  else if (d > ML_FIXED_1*ML_FIXED_1*8)
    d -= ML_FIXED_1*ML_FIXED_1*8;

  return (d >> ML_FSHIFT);
}


G_INLINE_FUNC MLFFloat ml_directb_diff (MLFFloat a, MLFFloat b)
{
  register MLFFloat diff;

  g_assert (a != ML_DIRECTB_UNKNOWN || b != ML_DIRECTB_UNKNOWN);

  diff = (a==ML_DIRECTB_UNKNOWN || b==ML_DIRECTB_UNKNOWN) ? 0 : ABS_DIFF (a,b);
  
  return (diff > 4) ? 8 - diff : diff;
}


G_INLINE_FUNC MLFFloat ml_directf_diff (MLFFloat a, MLFFloat b)
{
  register MLFFloat diff;

  g_assert (a != ML_DIRECTF_UNKNOWN || b != ML_DIRECTF_UNKNOWN);

  diff = (a==ML_DIRECTF_UNKNOWN || b==ML_DIRECTF_UNKNOWN) ? 0 : ABS_DIFF (a,b);

  return (diff > ML_FIXED_1 * 4) ? ML_FIXED_1 * 8 - diff : diff;
}
#endif	/* G_CAN_INLINE */


#define ML_LINE_INDEX_NO_LINE	0
#define ML_LINE_INDEX_CROSS	1

#define ML_PIC_SIZE(pic) ((pic)->cols*(pic)->rows)

#define ML_PIC_COORD(pic,row,col) ((pic)->cols*(row)+(col))
#define ML_PIC_COORD0(pic,coord) ((coord)-(pic)->cols)
#define ML_PIC_COORD1(pic,coord) ((coord)-1-(pic)->cols)
#define ML_PIC_COORD2(pic,coord) ((coord)-1)
#define ML_PIC_COORD3(pic,coord) ((coord)-1+(pic)->cols)
#define ML_PIC_COORD4(pic,coord) ((coord)+(pic)->cols)
#define ML_PIC_COORD5(pic,coord) ((coord)+1+(pic)->cols)
#define ML_PIC_COORD6(pic,coord) ((coord)+1)
#define ML_PIC_COORD7(pic,coord) ((coord)+1-(pic)->cols)

#define ML_PIC_PIXEL(pic,row,col) ((pic)->p[ML_PIC_COORD(pic,row,col)])
#define ML_PIC_PIXEL0(pic,coord) ((pic)->p[ML_PIC_COORD0(pic,coord)])
#define ML_PIC_PIXEL1(pic,coord) ((pic)->p[ML_PIC_COORD1(pic,coord)])
#define ML_PIC_PIXEL2(pic,coord) ((pic)->p[ML_PIC_COORD2(pic,coord)])
#define ML_PIC_PIXEL3(pic,coord) ((pic)->p[ML_PIC_COORD3(pic,coord)])
#define ML_PIC_PIXEL4(pic,coord) ((pic)->p[ML_PIC_COORD4(pic,coord)])
#define ML_PIC_PIXEL5(pic,coord) ((pic)->p[ML_PIC_COORD5(pic,coord)])
#define ML_PIC_PIXEL6(pic,coord) ((pic)->p[ML_PIC_COORD6(pic,coord)])
#define ML_PIC_PIXEL7(pic,coord) ((pic)->p[ML_PIC_COORD7(pic,coord)])

#define ML_PIC_COORD_ROW(pic,coord) ((coord)/(pic)->cols)
#define ML_PIC_COORD_COL(pic,coord) ((coord)%(pic)->cols)

/* 'thin': bit7 = line (skeleton), bit6-bit0: line's width
           bit6 = cleaned line, bit5-bit0: direct to  (FIXME: kurnik)

	   bit7 = bit6 = 0 => bit5-bit0: constants
*/
#define ML_THIN_LINE(pix) ((pix)&0x80)
#define ML_THIN_THIN(pix) (!((pix)&0x80) && ((pix)&0x40))

#define ML_THIN_LINE_SET(x) (0x80|(x))
#define ML_THIN_THIN_SET(x) (0x40|((x)&0x3f))

#define ML_THIN_LINE_GET(pix) ((pix)&0x7f)

#define ML_THIN_CONST_CROSS		0x01
#define ML_THIN_CONST_LINE_NO_LINE	0x02


/* picture.c */
MLPictureUI8*	ml_pic_new		(MLCoord	rows,
					 MLCoord	cols);
MLPictureSep*	ml_pic_read 		(gint 		fd);
#define		ml_pic_free(pic)	G_STMT_START{		\
			if (pic) { g_free (pic->p); g_free (pic); }	\
					}G_STMT_END
void		ml_pic_draw_lineAtom	(MLPictureUI8	*pic,
					 MLLineAtom	*atom,
					 guint8		 tips,
					 guint8		 body);		 
void		ml_pic_draw_line	(MLPictureUI8	*pic,
					 MLLine		*line,
					 guint8		 tips,
					 guint8		 body);		 

/* lfind.c */
MLPictureUI8*	ml_pic_thin 		(const MLPictureSep	*pic,
					 MLSeparations		 layer);
MLAtoms* 	ml_pic_find_atoms	(MLPictureUI8		*thin);


/* lally.c */
MLGlue*		ml_pic_glue		(const MLPictureSep	*pic,
					 const MLPictureUI8	*thin,
    					 MLAtoms		*atoms,
					 const MLGlueParam	*param,
					 MLGlue			*start);
MLGlue*		ml_pic_glue_default	(const MLPictureSep	*pic,
					 const MLPictureUI8	*thin,
    					 MLAtoms		*atoms,
					 MLSeparations		 layer);
guint		ml_pic_lines_connect	(const MLPictureSep	*pic,
    					 MLLines		*lines,
					 MLGlue			*glue,
					 const MLConnectParam	*param);

/* lineatom.c */
MLLineAtom*	ml_lineAtom_new		(guint			 len);
void		ml_lineAtom_free	(MLLineAtom		*atom);
MLFFloat	ml_lineAtom_direct	(const MLLineAtom	*atom,
					 guint8			 tip);
void		ml_lineAtom_reverse	(MLLineAtom		*atom);
MLTAtom		ml_lineAtom_classify	(const MLLineAtom	*atom,
					 const MLPictureUI8	*thin);


/* line.c */
MLLine*		ml_line_new		(MLLineAtom	*atom,
					 MLTLine	 type);
void		ml_line_free		(MLLine		*line);
MLFFloat	ml_line_direct		(const MLLine	*line,
					 guint8		 tip);
void		ml_line_reverse		(MLLine		*line);
MLFFloat	ml_line_quality		(const MLLine	*line);
void		ml_line_append		(MLLine		*line,
					 guint8		 tip,
					 MLLine		*line2,
					 guint8		 tip2,
					 MLLineAtom	*glue);


/* atoms.c */
MLAtoms*	ml_atoms_new		(void);
void		ml_atoms_free		(MLAtoms		*atoms);
void		ml_atoms_add_atom	(MLAtoms		*atoms,
					 MLLineAtom		*atom);
MLLines*	ml_atoms_to_lines	(MLAtoms		*atoms);
void		ml_atoms_classify	(MLAtoms		*atoms, 
					 const MLPictureSep	*pic,
					 const MLPictureUI8	*thin);

/* lines.c */
MLLines*	ml_lines_new 		(void);
void 		ml_lines_free 		(MLLines 	*lines);
void		ml_lines_add_line	(MLLines	*lines,
					 MLLine		*line);
void		ml_lines_stick		(MLLines	*lines,
					 MLLine		*line,
					 MLLine		*line2,
					 MLLineAtom	*glue,
					 MLTLine	 type);


/* glue.c */
MLGlue*		ml_glue_new		(void);
void		ml_glue_free		(MLGlue		*glue);
void		ml_glue_add_atom	(MLGlue		*glue,
					 MLLineAtom	*atom,
					 const MLAtoms	*atoms);
void		ml_glue_remove_atom	(MLGlue		*glue,
					 MLLineAtom	*atom);



/* penalty.c */
MLFFloat	ml_penalty_step_cross	(const MLStep	*curr, 
					 const MLSeparations	 pix,
					 guint8		 step_direct,
					 MLFFloat	 new_direct,
					 MLFFloat	 line_direct,
					 gboolean 	 is_cross);
MLFFloat	ml_penalty_step_over	(const MLStep	*curr, 
					 const MLSeparations	 pix,
					 guint8		 step_direct,
					 MLFFloat	 new_direct,
					 MLFFloat	 line_direct,
					 gboolean 	 is_cross);
MLFFloat	ml_penalty_step		(const MLStep	*curr, 
					 const MLSeparations	 pix,
					 guint8		 step_direct,
					 MLFFloat	 new_direct,
					 MLFFloat	 line_direct,
					 gboolean 	 is_cross);
MLFFloat	ml_penalty_connect	(const MLStep	*curr,
					 MLSeparations	 pix,
					 guint8		 step_direct,
					 MLFFloat	 new_direct,
					 MLFFloat	 line_direct,
					 gboolean	 is_cross,
					 const MLLineAtom *me,
					 guint8		 me_tip,
					 const MLLineAtom *neighbour,
					 guint8		 neighbour_tip);
#endif /* _M_ML_H */
