/*
 *   MAPA - MLAtoms structure manipulation
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: atoms.c,v 1.6 2001/04/19 15:32:32 mapa Exp $
 */

#include "ml.h"
#include "mlconfig.h"


MLAtoms *ml_atoms_new (void)
{
  MLAtoms *atoms;
  
  atoms = g_new (MLAtoms,1);
  atoms->atoms = g_ptr_array_new ();
  atoms->tips = g_hash_table_new (g_direct_hash, g_direct_equal);
  
  return atoms;
}


void ml_atoms_free (MLAtoms *atoms)
{
  gint i;
  
  for (i=0; i<atoms->atoms->len; i++)
    ml_lineAtom_free ((MLLineAtom*)g_ptr_array_index (atoms->atoms, i));

  g_ptr_array_free (atoms->atoms, TRUE);
  g_hash_table_destroy (atoms->tips);
}


void ml_atoms_add_atom (MLAtoms *atoms, MLLineAtom *atom)
{
  g_assert(!g_hash_table_lookup(atoms->tips, GUINT_TO_POINTER(atom->tip[0])));
  g_assert(!g_hash_table_lookup(atoms->tips, GUINT_TO_POINTER(atom->tip[1])));
      
  g_ptr_array_add (atoms->atoms, atom);
  g_hash_table_insert (atoms->tips, GUINT_TO_POINTER(atom->tip[0]), atom);
  g_hash_table_insert (atoms->tips, GUINT_TO_POINTER(atom->tip[1]), atom);
}



MLLines* ml_atoms_to_lines (MLAtoms *atoms)
{
  MLLineAtom *atom;
  MLLines *lines;
  guint i;
  MLTLine type;

  
  lines = ml_lines_new ();

  for (i = 0; i < atoms->atoms->len; i++) {
    atom = (MLLineAtom*) g_ptr_array_index (atoms->atoms, i);
        
    if (atom->type.line != MLTA_UNKNOWN && atom->type.line != MLTA_LINE)
      continue;

    /* simply clasify */
    type = MLTL_UNKNOWN;
    if (atom->len > 100)	/* FIXME */
      type = MLTL_LONG;
    
    ml_lines_add_line (lines, ml_line_new (atom, type));
  }
  
  return lines;
}



static guint16 neighbours (const MLLineAtom *atom,
    const MLAtoms *atoms, const MLPictureSep *pic,
    MLTAtom type)
{ 
  const MLLineAtom *atom2; 
  guint dots[8] = {0,0,0,0,0,0,0,0};
  guint16 dir;
  guint row,col, k,i, dr,dc;

  
  row = ML_PIC_COORD_ROW(pic,atom->tip[0]);
  col = ML_PIC_COORD_COL(pic,atom->tip[0]);
  
  for (i = 0; i < atoms->atoms->len; i++) {
    atom2 = (MLLineAtom*)g_ptr_array_index (atoms->atoms, i);
    
    if (atom == atom2 || atom2->len > 3 || atom2->type.line != type ||
	(dr = ABS_DIFF (row, ML_PIC_COORD_ROW(pic,atom2->tip[0]))) > 
	ML_RASTER_WINDOW  ||
	(dc = ABS_DIFF (col, ML_PIC_COORD_COL(pic,atom2->tip[0]))) > 
	ML_RASTER_WINDOW) 
      continue;
    
//    if (dr*dr + dc*dc > ML_RASTER_WINDOW*ML_RASTER_WINDOW)
//      continue;

    k = ((ML_PIC_COORD_COL(pic,atom2->tip[0]) > col) ? 4 : 0) +
      ((ML_PIC_COORD_ROW(pic,atom2->tip[0]) > row) ? 2 : 0) +
      ((dc > dr) ? 1 :0);
    
    dots[k]++;
  }
 
  dir = 0; 
  if (dots[0]) dir |= 1;
  if (dots[1]) dir |= 1<<1;
  if (dots[3]) dir |= 1<<2;
  if (dots[2]) dir |= 1<<3;
  if (dots[6]) dir |= 1<<4;
  if (dots[7]) dir |= 1<<5;
  if (dots[5]) dir |= 1<<6;
  if (dots[4]) dir |= 1<<7;

  if (dots[0] > 1) dir |= (1<<(1+8)) + (1<<(7+8));
  if (dots[1] > 1) dir |= (1<<(2+8)) + (1<<(0+8));
  if (dots[3] > 1) dir |= (1<<(3+8)) + (1<<(1+8));
  if (dots[2] > 1) dir |= (1<<(4+8)) + (1<<(2+8));
  if (dots[6] > 1) dir |= (1<<(5+8)) + (1<<(3+8));
  if (dots[7] > 1) dir |= (1<<(6+8)) + (1<<(4+8));
  if (dots[5] > 1) dir |= (1<<(7+8)) + (1<<(5+8));
  if (dots[4] > 1) dir |= (1<<(0+8)) + (1<<(6+8));
  
  return dir;
}



void classify_raster (MLAtoms *atoms,
    const MLPictureSep *pic, const MLPictureUI8 *thin)
{
  const MLLineAtom *atom;
  MLTAtom *arr;
  guint i,count;
  guint16 ne;
  
  arr = g_new (MLTAtom, atoms->atoms->len);

  /* zakl. test */
#ifdef DEBUG
  count = 0;
#endif
  for (i = 0; i < atoms->atoms->len; i++) {
    atom = (MLLineAtom*)g_ptr_array_index (atoms->atoms, i);
    
    ne = neighbours (atom, atoms, pic, MLTA_UNKNOWN);
    ne = (ne & 255) | (ne>>8);
    
    if (atom->len <= 3 && ne == 255 && atom->type.line == MLTA_UNKNOWN) {
      arr[i] = MLTA_RASTER;
#ifdef DEBUG
      count++;
#endif
    } else 
      arr[i] = atom->type.line;
  }
  
  for (i = 0; i < atoms->atoms->len; i++) 
    ((MLLineAtom*)g_ptr_array_index (atoms->atoms, i))->type.line = arr[i];
  
#ifdef DEBUG
  g_message ("RASTER: +%d", count);
  count = 0;
#endif
  

  /* odstraneni osamocenych rastru */
  for (i = 0; i < atoms->atoms->len; i++) {
    atom = (MLLineAtom*)g_ptr_array_index (atoms->atoms, i);
    
    ne = neighbours (atom, atoms, pic, MLTA_RASTER);
    ne = 
      ((ne&(1<<0))?1:0) +
      ((ne&(1<<1))?1:0) +
      ((ne&(1<<2))?1:0) +
      ((ne&(1<<3))?1:0) +
      ((ne&(1<<4))?1:0) +
      ((ne&(1<<5))?1:0) +
      ((ne&(1<<6))?1:0) +
      ((ne&(1<<7))?1:0);

    if (atom->len <= 3 && atom->type.line == MLTA_RASTER && ne < 2) {
      arr[i] = MLTA_UNKNOWN;
#ifdef DEBUG
      count++;
#endif
    } else 
      arr[i] = atom->type.line;
  }
  
  for (i = 0; i < atoms->atoms->len; i++) 
    ((MLLineAtom*)g_ptr_array_index (atoms->atoms, i))->type.line = arr[i];
  
#ifdef DEBUG
  g_message ("RASTER: -%d", count);
#endif

  /* expanze nalezenych oblasti */
  do {
    count = 0;
    
    for (i = 0; i < atoms->atoms->len; i++) {
      atom = (MLLineAtom*)g_ptr_array_index (atoms->atoms, i);

      ne = neighbours (atom, atoms, pic, MLTA_RASTER);
      ne = 
	((ne&(1<<0))?1:0) +
	((ne&(1<<1))?1:0) +
	((ne&(1<<2))?1:0) +
	((ne&(1<<3))?1:0) +
	((ne&(1<<4))?1:0) +
	((ne&(1<<5))?1:0) +
	((ne&(1<<6))?1:0) +
	((ne&(1<<7))?1:0);
      
      if (atom->len <= 3 && atom->type.line == MLTA_UNKNOWN && ne > 2) {
	arr[i] = MLTA_RASTER;
	count++;
      } else 
	arr[i] = atom->type.line;
    }
    for (i = 0; i < atoms->atoms->len; i++) 
      ((MLLineAtom*)g_ptr_array_index (atoms->atoms, i))->type.line = arr[i];

#ifdef DEBUG
    g_message ("RASTER: +%d", count);
#endif

  } while (count);
}



static void classify_notline (MLAtoms *atoms, 
    const MLPictureSep *pic, const MLPictureUI8 *thin)
{
  MLLineAtom *atom;
  guint i;

  for (i = 0; i < atoms->atoms->len; i++) {
    atom = (MLLineAtom*)g_ptr_array_index (atoms->atoms, i);
    
    if (atom->diff_width > 2*ML_FIXED_1 || 
	atom->width > ML_LINEWIDTH_MAX * ML_FIXED_1) 
      atom->type.line = MLTA_NOTLINE;
  }
}



void ml_atoms_classify (MLAtoms *atoms, 
    const MLPictureSep *pic, const MLPictureUI8 *thin)
{
#ifdef CLASSIFY_RASTER
  classify_raster (atoms, pic, thin);
#endif
  
  classify_notline (atoms, pic, thin);
}
