/*
 *   MAPA - connect default functions
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: connect.c,v 1.5 2001/04/19 15:34:43 mapa Exp $
 */

#include <stdlib.h>

#include "ml.h"
#include "mlconfig.h"

/* hodnotici funkce by mely byt soumerne vuci line<->line2 ! */
  
/*  LONG lines */
gboolean is_seed_long (const MLLine *line)
{
  return (line->len > 20);
}


gboolean is_possible_long (const MLLine *line, const MLLine *line2, 
    const MLLineAtom *gl, MLCoord line_tip, MLCoord line2_tip)
{
  if ((line->type != MLTL_UNKNOWN && line->type != MLTL_LONG) ||
      (line2->type != MLTL_UNKNOWN && line2->type != MLTL_LONG))
    return FALSE;
 
  /* stejne tluste */
  if (ABS_DIFF (line->width, line2->width) > (ML_FIXED_1/2)) return FALSE;
      
  switch (gl->type.glue) {
    case MLTG_SPACE:
      /* spojovat "pres prazdno" mohu jen dobre linie */ 
      if (gl->len > 30 || line->len < 50 || line2->len < 50) return FALSE;
      break; 
      
    case MLTG_OVERPRINT: 
      if (line->len + line2->len < 50 || line->len < 20 || line2->len < 20) 
	return FALSE;
      break;
      
    case MLTG_CROSS:
      /* zamezeni spojovani s chybami (drobna vetveni) zpusobene chybnou
	 klasifikaci */
      if (line->len < 5 || line2->len < 5) return FALSE;
      if (line->len + line2->len < 25) return FALSE;
      break;

    default: return FALSE;
  }
  
  return TRUE;
}



gboolean is_possible_cycle_long (const MLLine *line, const MLLine *line2,
    const MLLineAtom *gl, MLCoord line_tip, MLCoord gl_tip)
{
  g_assert (line2 == line);
  
  return line->len > 100 && gl->type.glue == MLTG_CROSS &&
      is_possible_long (line, line2, gl, line_tip, gl_tip);
}



MLLineAtom* pick_long (const MLLine *line, GPtrArray *gls,
    const MLLineAtom *gl2)
{
  g_assert (gls->len);
  
  if (gls->len == 1)
    return (MLLineAtom*)g_ptr_array_index (gls, 0);
  
  if (((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty * 2 
      < ((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty) {
#ifdef DEBUG_CONNECT
    g_message ("velky rozdit favorita (%.4f - %.4f) - BERU", 
	1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty / ML_FIXED_1, 
	1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty / ML_FIXED_1);
#endif
    
    return (MLLineAtom*)g_ptr_array_index (gls, 0);
  }
  
  return NULL;
}


/* DOTTED lines */

gboolean is_seed_dotted (const MLLine *line)
{
  return line->len <= 3 && line->width >= 1*ML_FIXED_1;
}


gboolean is_possible_dotted (const MLLine *line, const MLLine *line2, 
    const MLLineAtom *gl, MLCoord line_tip, MLCoord line2_tip)
{
  MLFFloat direct, direct2;
  guint8 tip;
  
  if ((line->type!=MLTL_DOTTED && (line->type!=MLTL_UNKNOWN||line->len > 3)) ||
      (line2->type!=MLTL_DOTTED && (line2->type!=MLTL_UNKNOWN||line2->len > 3)))
    return FALSE;

  if (line->width < 1*ML_FIXED_1 || line2->width < 1*ML_FIXED_1)
    /* zamezeni spojovani s chybami klasifikace ci ztencovani */
    return FALSE;
  
  if (gl->type.glue != MLTG_SPACE || 
      gl->len <= 3)	/* FIXME */
    return FALSE;
  
  tip = (line_tip == gl->tip[1]) ? 1 : 0;
  
  if (line->atoms->len == 1 && line2->atoms->len == 1)
    return ml_lineAtom_direct (gl,tip) != ML_DIRECTF_UNKNOWN;

  if (line->atoms->len > 1) {
    direct = ml_lineAtom_direct (gl,tip); 
    if (direct == ML_DIRECTF_UNKNOWN)
      return FALSE;

    direct = ml_directf_uturn (direct); 
    direct2 = ml_line_direct (line, line->tip[1] == line_tip ? 1 : 0);
    
    if (ml_directf_diff (direct, direct2) > ML_FIXED_1)
      return FALSE;
  }
  
  if (line2->atoms->len > 1) {
    direct = ml_lineAtom_direct (gl,tip?0:1); 
    if (direct == ML_DIRECTF_UNKNOWN)
      return FALSE;

    direct = ml_directf_uturn (direct); 
    direct2 = ml_line_direct (line2, line2->tip[1] == line2_tip ? 1 : 0);

    if (ml_directf_diff (direct, direct2) > ML_FIXED_1)
      return FALSE;
  }
  
  return TRUE;
}



gboolean is_possible_cycle_dotted (const MLLine *line, const MLLine *line2,
    const MLLineAtom *gl, MLCoord line_tip, MLCoord gl_tip)
{
  g_assert (line == line2);

  return line->atoms->len > 20 && 
      is_possible_dotted (line, line2, gl, line_tip, gl_tip);
}



MLLineAtom* pick_dotted (const MLLine *line, GPtrArray *gls,
    const MLLineAtom *gl2)
{
  guint i;

  g_assert (gls->len);
  
  /* velky naskok favorita */
  if (gls->len == 1)
    return (MLLineAtom*)g_ptr_array_index (gls, 0);
 
  if (1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty /
      ((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty < .7) {
#ifdef DEBUG_CONNECT
    g_message ("velky rozdit favorita (%.4f - %.4f) - BERU", 
	1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty / ML_FIXED_1, 
	1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty / ML_FIXED_1);
#endif
    
    return (MLLineAtom*)g_ptr_array_index (gls, 0);
  }


  /* mozna zacinam z prostred linie 
     (= dve dobre moznosti a dalsi mnohem horsi) */
  if (line->atoms->len == 1 &&
      (float)((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty /
      (float)((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty > .85 &&
      (gls->len > 2 &&
       1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty /
       ((MLLineAtom*)g_ptr_array_index (gls, 2))->penalty < .7)) {

    if (gl2 == (MLLineAtom*)g_ptr_array_index (gls, 1))
      return (MLLineAtom*)g_ptr_array_index (gls, 1);
    else 
      return (MLLineAtom*)g_ptr_array_index (gls, 0);
  }
  
  
  /* pokud je mezi dobrejma i gl2, tak beru */
  if (gl2) {
    for (i = 0; i < gls->len; i++) {
      if (1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty /
	  ((MLLineAtom*)g_ptr_array_index (gls, i))->penalty < .85)
	break;

      if (gl2 == (MLLineAtom*)g_ptr_array_index (gls, i))
	return (MLLineAtom*)g_ptr_array_index (gls, i);
    }
  }
  
  return NULL;
}

 
/*  DASHED lines */
gboolean is_seed_dash (const MLLine *line)
{
  return (line->len > 10 && line->len < 100);
}


#define myDIFF(x,y) (((x)&&(y)) ? 1.0*MIN(x,y)/MAX(x,y) : 1.0)

gboolean is_possible_dash (const MLLine *line, const MLLine *line2, 
    const MLLineAtom *gl, MLCoord line_tip, MLCoord line2_tip)
{
  if ((line->type != MLTL_UNKNOWN && line->type != MLTL_DASHED) ||
      (line2->type != MLTL_UNKNOWN && line2->type != MLTL_DASHED))
    return FALSE;
 
  /* stejne tluste */
  if (ABS_DIFF (line->width, line2->width) > ML_FIXED_1/2) return FALSE;
 
  switch (gl->type.glue) {
    case MLTG_CROSS:
      if (line->pieces || line2->pieces ||
	  line->len < 5 || line2->len < 5 ||
	  line->len + line2->len < 20) return FALSE;
      break;

    case MLTG_SPACE:
      if (line->len < 10 ||  line2->len < 10 || 
	  myDIFF(line->avg_black,line2->avg_black) < .8 ||
	  myDIFF(line->avg_space,line2->avg_space) < .8 ||
	  myDIFF(line->avg_space,gl->len) < .8 ||
	  myDIFF(line2->avg_space,gl->len) < .8)
	return FALSE;
      break; 
      
    case MLTG_OVERPRINT: 
      if (line->pieces < 5 || line2->pieces < 5 ||
	  line->len < 10 || line2->len < 10 ||
	  myDIFF(line->avg_black,line2->avg_black) < .9 ||
	  myDIFF(line->avg_space,line2->avg_space) < .9 ||
	  myDIFF(line->avg_space,gl->len) < .8 ||
	  myDIFF(line2->avg_space,gl->len) < .8) return FALSE;
      break;
 

    default: return FALSE;
  }
  
  
  return TRUE;
}



gboolean is_possible_cycle_dash (const MLLine *line, const MLLine *line2,
    const MLLineAtom *gl, MLCoord line_tip, MLCoord gl_tip)
{
  g_assert (line2 == line);
  
  return line->len > 100 && gl->type.glue == MLTG_CROSS && line->pieces > 2 &&
      is_possible_dash (line, line2, gl, line_tip, gl_tip);
}



MLLineAtom* pick_dash (const MLLine *line, GPtrArray *gls,
    const MLLineAtom *gl2)
{
  g_assert (gls->len);
  
  if (gls->len == 1)
    return (MLLineAtom*)g_ptr_array_index (gls, 0);
  
  if (((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty * 2 
      < ((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty) {
#ifdef DEBUG_CONNECT
    g_message ("velky rozdit favorita (%.4f - %.4f) - BERU", 
	1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 0))->penalty / ML_FIXED_1, 
	1.0 * ((MLLineAtom*)g_ptr_array_index (gls, 1))->penalty / ML_FIXED_1);
#endif
    
    return (MLLineAtom*)g_ptr_array_index (gls, 0);
  }
  
  return NULL;
}


const MLConnectParam default_connect_param[] = {
  { is_seed_dash, is_possible_dash, is_possible_cycle_dash, pick_dash, 
    MLTL_DASHED },
  { is_seed_long, is_possible_long, is_possible_cycle_long, pick_long,
    MLTL_LONG },
  { is_seed_dotted, is_possible_dotted, is_possible_cycle_dotted, pick_dotted,
    MLTL_DOTTED }, 
  { NULL, NULL, NULL, NULL, MLTL_UNKNOWN }
};
