/*
 *   MAPA - MLGlue manipulation
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: glue.c,v 1.5 2001/04/11 11:58:42 mapa Exp $
 */

#include <string.h>

#include "ml.h"


MLGlue *ml_glue_new (void)
{
  MLGlue *glue;
  
  glue = g_new (MLGlue,1);
  glue->atoms = g_ptr_array_new ();
  glue->tips = g_hash_table_new (g_direct_hash, g_direct_equal);
  
  return glue;
}


static void fg (gpointer key, gpointer value, gpointer user_data)
{
  g_ptr_array_free ((GPtrArray*)value, TRUE);
}


void ml_glue_free (MLGlue *glue)
{
  gint i;
  
  g_hash_table_foreach (glue->tips, fg, NULL);
  g_hash_table_destroy (glue->tips);
  
  for (i=0; i<glue->atoms->len; i++)
    ml_lineAtom_free ((MLLineAtom*)g_ptr_array_index (glue->atoms, i));
  g_ptr_array_free (glue->atoms, TRUE);
}


static void add_atom (GPtrArray *arr, MLLineAtom *atom)
{
  guint i;
    
  for (i = 0; i < arr->len; i++) {
    if (((MLLineAtom*)g_ptr_array_index(arr,i))->penalty > atom->penalty)
      break;
  }
  
  if (i == arr->len) {
    g_ptr_array_add (arr, atom);
    return;
  }
  
  g_assert (i < arr->len);
 
  g_ptr_array_add (arr, NULL);	/* expand array */

  g_memmove (arr->pdata + i + 1, arr->pdata + i, 
      sizeof (gpointer) * (arr->len - i - 1));
  
  g_ptr_array_index (arr,i) = atom;
}


/* prida atom do glue, pokud pro dannou cestu existuje dalsi varianta,
   ulozi tu lepsi. U kratkych linii nezalezi, ke kteremu konci
   cesta vede! */
void ml_glue_add_atom (MLGlue *glue, MLLineAtom *atom, const MLAtoms *atoms)
{
  GPtrArray *gls;
  const MLLineAtom *atom0, *atom1;
  MLLineAtom *rival;
  MLCoord atom0_tipA, atom0_tipB, atom1_tipA, atom1_tipB;
  gint i;
  
  gls = (GPtrArray*)
    g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom->tip[0]));
  
  atom0 = (MLLineAtom*)
    g_hash_table_lookup (atoms->tips, GUINT_TO_POINTER(atom->tip[0]));
  atom1 = (MLLineAtom*)
    g_hash_table_lookup (atoms->tips, GUINT_TO_POINTER(atom->tip[1]));
  
  g_assert (atom0 && atom1);
 

  atom0_tipA = atom->tip[0];
  atom1_tipA = atom->tip[1];
  
  /* na (0,0) zadna linie zacinat nemuze => tip = 0 == nothing */
  atom0_tipB = (atom0->len > 3) ? 0 : 
    (atom0->tip[0] == atom->tip[0] ? atom0->tip[1] : atom0->tip[0]);
  atom1_tipB = (atom1->len > 3) ? 0 : 
    (atom1->tip[0] == atom->tip[1] ? atom1->tip[1] : atom1->tip[0]);
  
  /* try to find an atom on the same path */
  rival = NULL;
  gls = (GPtrArray*)
    g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom0_tipA));

  if (gls) {
    for (i = 0; i < gls->len; i++) {
      rival = ((MLLineAtom*)g_ptr_array_index (gls, i));
     
      if ((rival->tip[0] == atom0_tipA && 
	    (rival->tip[1] == atom1_tipA || rival->tip[1] == atom1_tipB)) ||
	  (rival->tip[1] == atom0_tipA && 
	    (rival->tip[0] == atom1_tipA || rival->tip[0] == atom1_tipB)))
	  break;
      
      rival = NULL;
    }
  }
 

  if (rival == NULL && atom0->len <= 3) {
    /* musim prozkoumat i druhy konec line0 */
    gls = (GPtrArray*)
      g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom0_tipB));

    if (gls) {
      for (i = 0; i < gls->len; i++) {
	rival = ((MLLineAtom*)g_ptr_array_index (gls, i));
     
	if ((rival->tip[0] == atom0_tipB && 
	      (rival->tip[1] == atom1_tipA || rival->tip[1] == atom1_tipB)) ||
	    (rival->tip[1] == atom0_tipB && 
	     (rival->tip[0] == atom1_tipA || rival->tip[0] == atom1_tipB)))
	  break;

	rival = NULL;
      }
    }
  }
  
  if (rival != NULL) {
    /* is the rival better ? */
    
    if (rival->penalty <= atom->penalty) {
      ml_lineAtom_free (atom);
      
      return;
    }
    
    ml_glue_remove_atom (glue, rival);
    ml_lineAtom_free (rival);
  }
  
 
  /* add the atom */
  g_ptr_array_add (glue->atoms, atom);
 
  gls = (GPtrArray*)
    g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom->tip[0]));
  if (! gls) {
    gls = g_ptr_array_new ();
    g_hash_table_insert (glue->tips, GUINT_TO_POINTER(atom->tip[0]), gls);
  }
  
  add_atom (gls, atom);

  gls = (GPtrArray*)
    g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom->tip[1]));
  if (! gls) {
    gls = g_ptr_array_new ();
    g_hash_table_insert (glue->tips, GUINT_TO_POINTER(atom->tip[1]), gls);
  }

  add_atom (gls, atom);
}



void ml_glue_remove_atom (MLGlue *glue, MLLineAtom *atom)
{
  gboolean ret;
  
  ret = g_ptr_array_remove_fast (glue->atoms, atom);
  g_assert (ret);
  
  ret = g_ptr_array_remove ((GPtrArray*)
      g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom->tip[0])), atom);
  g_assert (ret);

  ret = g_ptr_array_remove ((GPtrArray*)
      g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(atom->tip[1])), atom);
  g_assert (ret);
}
