/*
 *   MAPA - friendships between lines
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: lally.c,v 1.37 2001/04/11 12:00:35 mapa Exp $
 */

#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "ml.h"
#include "mlconfig.h"

#ifdef DEBUG
#include "mlt.h"
#endif

/* smery:
   -------------        -------------
   | \ | | | / |        | 1 | 0 | 7 | 
   -------------        -------------
   | - | O | - |        | 2 | O | 6 |
   -------------        -------------
   | / | | | \ |        | 3 | 4 | 5 |
   -------------        -------------
 */


#define CONNECT_QUALITY_PERFECT 64

#define CONN_BAD_GET(pix)	((pix)&0x80)
#define CONN_BAD_SET(pix,x)	(((pix)&0x7f)|(0x80*(x)))
#define CONN_CONST_GET(pix)	((pix)&0x7f)
#define CONN_CONST_CROSS	0x20
#define CONN_CONST_LINE		0x7f
#define CONN_CONST_TIP		0x50

static void next_step (MLPictureUI8 *conn, const MLPictureSep *pic, 
    const MLAtoms *atoms, 
    const MLLineAtom *atom, guint8 atom_tip,
    GPtrArray *steps, GPtrArray *sort, GHashTable *active, 
    MLStep *curr, MLCoord new_coord, guint8 step_direct,
    const MLGlueParam *param,
    GMemChunk *mem) 
{
  MLFFloat direct;
  MLFFloat penalty;
  MLStep *new;
  const MLLineAtom *neighbour;
  guint j;
  
  if (CONN_BAD_GET(conn->p[new_coord]) || 
      CONN_CONST_GET(conn->p[new_coord]) == CONN_CONST_LINE) return; 

  direct = ml_directf_calc (curr->direct, ML_FIXED_1 * step_direct, 
      ML_DIRECT_STEP_EXP); 
 
  penalty = curr->penalty;
 
  if (CONN_CONST_GET(conn->p[new_coord]) != CONN_CONST_TIP) 
    penalty += 
      param->step_penalty(curr, pic->p[new_coord], step_direct, direct,
	  ((MLStep*)g_ptr_array_index (steps,0))->direct,
	  CONN_CONST_GET(conn->p[new_coord]) == CONN_CONST_CROSS);
  else {
    neighbour = (MLLineAtom*)
      g_hash_table_lookup (atoms->tips, GUINT_TO_POINTER(new_coord));

    penalty += 
      param->connect_penalty (curr, pic->p[new_coord], step_direct, direct,
	  ((MLStep*)g_ptr_array_index (steps,0))->direct,
	  CONN_CONST_GET(conn->p[new_coord]) == CONN_CONST_CROSS,
	  atom, atom_tip,
	  neighbour, (neighbour->tip[1] == new_coord) ? 1 : 0);
  }


  /* neznam sem lepsi cestu ? */
  new = (MLStep*)g_hash_table_lookup (active, GUINT_TO_POINTER(new_coord));

  if (new == NULL) { /* novy */
    g_ptr_array_set_size (steps, steps->len+1);
    g_ptr_array_index (steps, steps->len-1) = new = g_chunk_new (MLStep,mem);
    
    g_hash_table_insert (active, GUINT_TO_POINTER(new_coord), new); 

  } else if (new->penalty > penalty) { /* nalezl jsem lepsi cestu */ 
    g_ptr_array_remove (sort, new);

  } else { /* uz jsem tu byl lepsi cestou */
    return; 
  }

  new->coord = new_coord;
  new->direct = direct;
  new->penalty = penalty;
  new->direct_in = step_direct;
  new->len = curr->len + 1;
  new->in = curr;
  
  if (param->debug != NULL)
    param->debug->p[new_coord] = (255 * penalty) / param->max_penalty;
  
  /* najdu misto, kam patrim 
     FIXME: logaritm. vyhledavani a memcpy by bylo asi rychlejsi */ 
  g_ptr_array_set_size (sort, sort->len+1);
  for (j = sort->len-1; j > 0; j--) {
    if (((MLStep*)g_ptr_array_index(sort,j-1))->penalty < penalty) 
      break;

    g_ptr_array_index (sort,j) = g_ptr_array_index (sort, j-1);
  }
  
  g_ptr_array_index (sort, j) = new;
}



static void line_neighbours (MLPictureUI8 *conn,
    const MLPictureSep *pic, const MLAtoms *atoms, 
    const MLLineAtom *atom, guint8 tip, 
    const MLGlueParam *param, MLGlue *glue)
{
  GPtrArray *steps;	/* stare i aktivni kroky */
  GPtrArray *sort;	/* srovnane odkazy na aktivni pixely */
  GHashTable *active;	/* odkazy na pixely, ktere jsem zatim neprosel, ale 
			   uz navstivil a jsou "pruchozi". Klicem je coord */
  MLLineAtom *out;
  MLStep *curr, *tmp;
  MLFFloat width;
  guint i;
  gboolean notfirst = FALSE;
  GMemChunk *mem;

  /* init */
  width = atom->width;
  steps = g_ptr_array_new ();
  sort = g_ptr_array_new ();
  active = g_hash_table_new (NULL, NULL);
  mem = g_mem_chunk_create (MLStep, 2048, G_ALLOC_ONLY);
  
  curr = g_chunk_new (MLStep, mem);
  
  curr->coord = atom->tip[tip];
  curr->direct = ml_lineAtom_direct (atom, atom->len * tip);  
  curr->penalty = 0;
  curr->direct_in = 0;
  curr->len = 0;
  curr->in = NULL;
  
  g_ptr_array_add (steps, curr);
  g_ptr_array_add (sort, curr);

  /* run */
  while (curr->penalty < param->max_penalty) {
    /* ostran curr z active */
    g_hash_table_remove (active, GUINT_TO_POINTER(curr->coord));
    g_ptr_array_remove_index (sort, 0);	
    conn->p[curr->coord] = CONN_BAD_SET (conn->p[curr->coord],1);

    if (CONN_CONST_GET(conn->p[curr->coord]) == CONN_CONST_TIP && 
	notfirst) {
   
      /* narazil jsem na jinou linii, vytvor spoj */
      out = ml_lineAtom_new (curr->len);
      out->tip[0] = atom->tip[tip];
      out->tip[1] = curr->coord;
      out->penalty = curr->penalty;
      out->type.glue = param->glue_type;
      
      i = curr->len;
      tmp = curr;
      while (i--) {
	out->data[i] = tmp->direct_in;
	tmp = tmp->in;
      }
      g_assert (tmp->in == NULL);
      
      ml_glue_add_atom (glue, out, atoms);
      
    } else {
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr, 
	  ML_PIC_COORD0(pic,curr->coord), 0, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD1(pic,curr->coord), 1, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD2(pic,curr->coord), 2, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD3(pic,curr->coord), 3, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD4(pic,curr->coord), 4, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD5(pic,curr->coord), 5, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD6(pic,curr->coord), 6, param, mem);
      next_step (conn, pic, atoms, atom, tip, steps, sort, active, curr,
	  ML_PIC_COORD7(pic,curr->coord), 7, param, mem);
    }

    if (! sort->len) 
      /* nemam kam jit */
      break;

    curr = (MLStep*)g_ptr_array_index(sort,0);

    notfirst = TRUE;
  }
  

  /* uklid */
  for (i = 0; i < steps->len; i++) {
    curr = g_ptr_array_index (steps, i);

    if (g_hash_table_lookup (active, GUINT_TO_POINTER(curr->coord)) == NULL) {
      g_assert (CONN_BAD_GET(conn->p[curr->coord]));
      conn->p[curr->coord] = CONN_BAD_SET(conn->p[curr->coord],0);
    }
  }

  g_ptr_array_free (steps, TRUE);
  g_ptr_array_free (sort, TRUE);
  g_hash_table_destroy (active);
  g_mem_chunk_destroy (mem);
}



static MLCoord conn_draw_lineAtom (MLPictureUI8 *conn, MLLineAtom *atom) 
{
  MLCoord coord;
  guint i;
  
  coord = atom->tip[0];
  
  conn->p[coord] = CONN_CONST_TIP;

  for (i=0; i < atom->len; i++) {
    if (i != 0 && i != atom->len) {
      conn->p[coord] = CONN_CONST_LINE;

      switch (atom->data[i]) {
	case 1: ML_PIC_PIXEL0(conn,coord) = CONN_CONST_LINE;
		ML_PIC_PIXEL2(conn,coord) = CONN_CONST_LINE;
		break;
	case 3: ML_PIC_PIXEL2(conn,coord) = CONN_CONST_LINE;
		ML_PIC_PIXEL4(conn,coord) = CONN_CONST_LINE;
		break;
	case 5: ML_PIC_PIXEL4(conn,coord) = CONN_CONST_LINE;
		ML_PIC_PIXEL6(conn,coord) = CONN_CONST_LINE;
		break;
	case 7: ML_PIC_PIXEL6(conn,coord) = CONN_CONST_LINE;
		ML_PIC_PIXEL0(conn,coord) = CONN_CONST_LINE;
		break;
      }
    }

    switch (atom->data[i]) {
      case 0: coord = ML_PIC_COORD0(conn,coord);
	      break;
      case 1: coord = ML_PIC_COORD1(conn,coord);
	      break;
      case 2: coord = ML_PIC_COORD2(conn,coord);
	      break;
      case 3: coord = ML_PIC_COORD3(conn,coord);
	      break;
      case 4: coord = ML_PIC_COORD4(conn,coord);
	      break;
      case 5: coord = ML_PIC_COORD5(conn,coord);
	      break;
      case 6: coord = ML_PIC_COORD6(conn,coord);
	      break;
      case 7: coord = ML_PIC_COORD7(conn,coord);
	      break;
      default: g_assert_not_reached ();
    }
  }
  
  g_assert (coord == atom->tip [1]);

  conn->p[coord] = CONN_CONST_TIP;

  return coord;
}



MLPictureUI8* conn_new (const MLPictureUI8 *thin, const MLAtoms *atoms)
{
  register MLCoord coord;
  MLPictureUI8* conn;
  gint i;
  
#ifdef DEBUG
  g_message ("inicializuji 'conn' (lines:%d)...", atoms->atoms->len);
#endif
  
  conn = g_new (MLPictureUI8, 1);
  conn->rows = thin->rows;
  conn->cols = thin->cols;
  conn->p = g_new0 (guint8, ML_PIC_SIZE(conn));
  
  /* oramovani */
  for (coord = 0; coord < conn->cols; coord++)
    conn->p[coord] = CONN_BAD_SET (0,1);
  
  for (coord = ML_PIC_COORD(conn,conn->rows-1,0); 
       coord < ML_PIC_SIZE(conn); coord++)
    conn->p[coord] = CONN_BAD_SET (0,1);
  
  for (coord = 0; coord < ML_PIC_SIZE(conn); coord += conn->cols) {
    conn->p[coord] = CONN_BAD_SET (0,1);
    conn->p[coord+conn->cols-1] = CONN_BAD_SET (0,1);
  }
  
  
  /* prekresleni krizovatek */
  for (coord = 0; coord < ML_PIC_SIZE(conn); coord++)
    if (thin->p[coord] == ML_THIN_CONST_CROSS)
      conn->p[coord] = CONN_CONST_CROSS;

  
  /* vykresleni linii */
  for (i = 0; i < atoms->atoms->len; i++) 
    conn_draw_lineAtom (conn, (MLLineAtom*)g_ptr_array_index (atoms->atoms, i));
      
#ifdef DEBUG
  g_message ("   ... hotovo");
#endif
  
  return conn;
}


MLGlue *ml_pic_glue (const MLPictureSep *pic, const MLPictureUI8 *thin,
    MLAtoms *atoms, const MLGlueParam *param, MLGlue *glue)
{
  MLPictureUI8 *conn;
  MLLineAtom *atom;
  guint8 tip;
  guint i;
  
#ifdef DEBUG
  g_message ("Propojuji lines ...");
#endif
  
  g_assert (param);
 
  if (! glue) glue = ml_glue_new ();

  conn = conn_new (thin, atoms);
  
  for (i = 0, tip = 0; i < atoms->atoms->len; i += tip, tip = tip?0:1) {
#ifdef DEBUG
    if (!tip) g_print (".");
#endif
    
    atom = (MLLineAtom*)g_ptr_array_index (atoms->atoms, i);
    
    if (atom->type.line != MLTA_UNKNOWN && atom->type.line != MLTA_LINE)
      continue;

    if (ML_PIC_COORD_ROW(thin,atom->tip[tip]) > 5 &&
	ML_PIC_COORD_COL(thin,atom->tip[tip]) > 5 &&
	ML_PIC_COORD_ROW(thin,atom->tip[tip]) < thin->rows - 5 &&
	ML_PIC_COORD_COL(thin,atom->tip[tip]) < thin->cols - 5)
      line_neighbours (conn, pic, atoms, atom, tip, param, glue);
  }
  
  g_print ("\n");
 
  ml_pic_free (conn);
  
  return glue;
}


MLGlue *ml_pic_glue_default (const MLPictureSep *pic, const MLPictureUI8 *thin,
    MLAtoms *atoms, MLSeparations layer)
{
  MLGlue *glue;
  MLGlueParam param;
 
  /* cross */ 
#ifdef DEBUG
  g_message (" cross-glue ");
#endif
  param.step_penalty = ml_penalty_step_cross;
  param.connect_penalty = ml_penalty_connect;
  param.max_penalty = ML_FIXED_1 / 16;
  param.glue_type = MLTG_CROSS;
  param.debug = NULL;
    
  glue = ml_pic_glue (pic, thin, atoms, &param, NULL);


  /* overprint */
  if (!(layer & ML_LAYER_BK)) {
#ifdef DEBUG
    g_message (" overprint-glue ");
#endif
    param.step_penalty = ml_penalty_step_over;
    param.max_penalty = ML_FIXED_1 / 4;
    param.glue_type = MLTG_OVERPRINT;
    
    glue = ml_pic_glue (pic, thin, atoms, &param, glue);
  }
  
  
  /* space */
#ifdef DEBUG
  g_message (" space-glue ");
#endif
  param.step_penalty = ml_penalty_step;
  param.max_penalty = ML_FIXED_1;
  param.glue_type = MLTG_SPACE;
    
  glue = ml_pic_glue (pic, thin, atoms, &param, glue);
  
  return glue;
}
