/*
 *   MAPA - connections between lines
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: lconn.c,v 1.8 2001/04/19 15:35:20 mapa Exp $
 */

#include "ml.h"

#ifdef DEBUG
const MLPictureSep *p;
#endif


extern const MLConnectParam default_connect_param[];


/* return array is sorted !! */
static GPtrArray* find_gl (const MLLine *line, const MLCoord line_tip, 
    const MLLines *lines, const MLGlue *glue, const MLConnectParam *param)
{
  GPtrArray *arr, *goods;
  MLLineAtom *gl;
  MLLine *line2;
  MLCoord gl_tip;
  guint i;

  g_assert (line->tip[0] == line_tip || line->tip[1] == line_tip);

  arr = g_hash_table_lookup (glue->tips, GUINT_TO_POINTER(line_tip));
  
  if (arr == NULL) 
    return NULL;
 
  goods = NULL;
  
  /* this is for happy compiler only */
  gl = NULL;
  
  for (i = 0; i < arr->len; i++) { 
    /* nalezeni sousedky */
    gl = (MLLineAtom*)g_ptr_array_index (arr, i);
    if (gl->line != NULL) {
      /* mozne jedine pokud posledni atom linie ma delku 0 
	 (pak jeho tip[0] == tip[1]) */
      g_assert (((MLLineAtom*)g_ptr_array_index (line->atoms, 
	      (line->tip[1] == line_tip) ? line->atoms->len-1 : 0))->len == 0);
      g_assert (gl->line == line);

      continue;
    }

    gl_tip = (line_tip == gl->tip[0]) ? gl->tip[1] : gl->tip[0];
    g_assert (gl->tip[0] == line_tip || gl->tip[1] == line_tip);

    line2 = (MLLine*)g_hash_table_lookup(lines->tips, GUINT_TO_POINTER(gl_tip));
    if (line2 == NULL) 
      /* FIXME: connect */
      continue;

    if (
	(line2 == line &&
	 param->is_possible_cycle (line, line2, gl, line_tip, gl_tip)) ||
	(line2 != line &&
	 param->is_possible (line, line2, gl, line_tip, gl_tip))) {

      if (!goods) 
	goods = g_ptr_array_new ();

      g_ptr_array_add (goods, gl);

#ifdef DEBUG_CONNECT
      g_message("%s%d:(%d,%d)-%d/%d(%d)-(%d,%d) =%c%d(%.3f)= %d:(%d,%d)-%d/%d(%d)-(%d,%d)",
	  line == line2 ? "C" : "",
	  line_tip == line->tip[1] ? 1 : 0,
	  ML_PIC_COORD_ROW(p,line->tip[0]),
	  ML_PIC_COORD_COL(p,line->tip[0]),
	  line->len,
	  line->atoms->len,
	  line->width,
	  ML_PIC_COORD_ROW(p,line->tip[1]),
	  ML_PIC_COORD_COL(p,line->tip[1]),

	  (gl->type.glue == MLTG_SPACE) ? 'S' : (gl->type.glue == MLTG_OVERPRINT) ? 'P' : (gl->type.glue == MLTG_CROSS) ? 'C' : '?',
	  gl->len,
	  1.0 * gl->penalty / ML_FIXED_1,

	  gl_tip == line2->tip[1] ? 1 : 0,
	  ML_PIC_COORD_ROW(p,line2->tip[0]),
	  ML_PIC_COORD_COL(p,line2->tip[0]),
	  line2->len,
	  line2->atoms->len,
	  line2->width,
	  ML_PIC_COORD_ROW(p,line2->tip[1]),
	  ML_PIC_COORD_COL(p,line2->tip[1]));
#endif
    }
  }

#ifdef DEBUG_CONNECT
  if (goods) 
    g_message ("good_count: %d", goods->len);
#endif
  
  return goods;
}


static guint expand_line (MLLine *line, guint8 tip, 
    MLLines *lines, MLGlue *glue, const MLConnectParam *param)
{
  MLLineAtom *gl, *gl2;
  GPtrArray *gls;
  MLLine *line2;
  MLCoord line_tip, gl_tip;
  
  if (line->cycle)
    return 0;
 
  gls = find_gl (line, line->tip[tip], lines, glue, param);
  if (gls == NULL) return 0;
  
  gl = param->pick (line, gls, NULL);
  g_ptr_array_free (gls, TRUE);
  if (gl == NULL) return 0;
  
  line_tip = line->tip[tip];
  gl_tip = (line_tip == gl->tip[0]) ? gl->tip[1] : gl->tip[0];
  
  line2 = (MLLine*)g_hash_table_lookup(lines->tips, GUINT_TO_POINTER(gl_tip));
  g_assert (line2);
  
  gls = find_gl (line2, gl_tip, lines, glue, param);
  if (gls == NULL) return 0; 

  gl2 = param->pick (line2, gls, gl);
  g_ptr_array_free (gls, TRUE);
  
  if (gl != gl2)
    return 0;
  
#ifdef DEBUG_CONNECT
  g_message(" ... spojuji %c%d(%.3f)",
       (gl->type.glue == MLTG_SPACE) ? 'S' : (gl->type.glue == MLTG_OVERPRINT) ? 'P' : (gl->type.glue == MLTG_CROSS) ? 'C' : '?',
       gl->len,
       1.0 * gl->penalty / ML_FIXED_1);
#endif
  
  ml_lines_stick (lines, line, line2, gl, param->type);
  
  return 1;
}


guint ml_pic_lines_connect (const MLPictureSep *pic,
    MLLines *lines, MLGlue *glue, 
    const MLConnectParam *param)
{
  MLLine *line;
  guint succeeds, tip0, tip1;
  guint i, j;
 
#ifdef DEBUG
  p = pic; 
  g_message ("spojuji linie ...");
#endif

  if (param == NULL)
    param = default_connect_param;
  
  succeeds = 0;

  /* the order of param[] is important !! It's linetype-priority */
  for (j = 0; param[j].type != MLTL_UNKNOWN; j++) {
    for (i = 0; i < lines->lines->len; i++) {
      line = (MLLine*) g_ptr_array_index (lines->lines, i);

#ifdef DEBUG_CONNECT
      g_message ("zpracovavam (%d,%d)-%d(%d)-(%d,%d)",
	  ML_PIC_COORD_ROW(p,line->tip[0]),
	  ML_PIC_COORD_COL(p,line->tip[0]),
	  line->len,
	  line->width,
	  ML_PIC_COORD_ROW(p,line->tip[1]),
	  ML_PIC_COORD_COL(p,line->tip[1]));
#else
#ifdef DEBUG
      g_print (".");
#endif
#endif
    
      tip0 = tip1 = 1;

      if (line->type == MLTL_UNKNOWN) {
	if (param[j].is_seed (line)) {
	  /* try to expand */
	  succeeds += tip0 = expand_line (line, 0, lines, glue, &param[j]);
	  succeeds += tip1 = expand_line (line, 1, lines, glue, &param[j]);
	}
      }
     
       
      if (line->type == param[j].type) {
	/* hura - pekne zatlacime na pilu */
	while (tip0 || tip1) {
	  if (tip0) 
	    succeeds += tip0 = expand_line (line, 0, lines, glue, &param[j]);

	  if (tip1) 
	    succeeds += tip1 = expand_line (line, 1, lines, glue, &param[j]);
	}
      }
    }
   

    if (param[j].type == MLTL_DASHED) {
      /* carkovane cary s prilis malo kousky jsou spis plne cary ! */
      for (i = 0; i < lines->lines->len; i++) {
	line = (MLLine*)g_ptr_array_index (lines->lines, i);

	if (line->type == MLTL_DASHED && line->pieces < 2)
	  line->type = MLTL_LONG;
      }
    }
  }
  
#ifdef DEBUG
  g_message (" ... hotovo");
#endif

  return succeeds;
}
