/*
 *   MAPA - basic tools for manipulation with a lineAtom
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: lineatom.c,v 1.4 2001/04/11 12:00:35 mapa Exp $
 */

#include <unistd.h>

#include "ml.h"
#include "mlconfig.h"


MLLineAtom* ml_lineAtom_new (guint len)
{
  MLLineAtom *atom;
  
  atom = g_new0 (MLLineAtom, 1);
  atom->len = len;
  atom->data = g_new0 (guint8, len);
  
  return atom;
}


void ml_lineAtom_free (MLLineAtom *atom)
{
  if (atom) {
    g_free (atom->data);
    g_free (atom);
  }
}


MLFFloat ml_lineAtom_direct (const MLLineAtom *atom, guint8 tip)
{
  MLFFloat direct;
  guint i;
  
  if (atom->len < (ML_DIRECT_LINE_LEN+1)/2)
    /* u kratkych linii je spocteny smer irelevantni (= na hovno) */
    return ML_DIRECTF_UNKNOWN;
  
  if (tip) {
    i = (ML_DIRECT_LINE_LEN > atom->len) ? 0 : atom->len - ML_DIRECT_LINE_LEN;
    
    direct = ML_FIXED_1 * atom->data[i];
    
    do {
      direct = ml_directf_calc (direct,
	  ML_FIXED_1 * atom->data[i],
	  ML_DIRECT_LINE_EXP);
    } while (++i < atom->len);
  
  } else {

    i = MIN (ML_DIRECT_LINE_LEN, atom->len);
    
    direct = ML_FIXED_1 * ml_directb_uturn (atom->data[i-1]);
    
    while (i--) 
      direct = ml_directf_calc (direct,
	  ML_FIXED_1 * ml_directb_uturn (atom->data[i]),
	  ML_DIRECT_LINE_EXP);
  }
  
  return direct;
}


void ml_lineAtom_reverse (MLLineAtom *atom)
{
  guint8 *new_data;
  register int i,j;
  MLCoord coord;
  
  if (!atom->len) return;
  
  coord = atom->tip[0];
  atom->tip[0] = atom->tip[1];
  atom->tip[1] = coord;
  
  new_data = g_new (guint8, atom->len);
  
  for (i=0,j=atom->len-1; j >= 0; i++,j--)
    new_data[i] = ml_directb_uturn (atom->data[j]);
  
  g_free (atom->data);
  atom->data = new_data;
}


MLTAtom ml_lineAtom_classify (const MLLineAtom *atom, const MLPictureUI8 *thin)
{
  /* FIXME: dodelat */
  
  return MLTA_UNKNOWN;
}
