/*
 *   MAPA - basic tools for manipulation with a lines
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: lines.c,v 1.18 2001/04/04 10:02:32 mapa Exp $
 */

#include <unistd.h>
#include <stdio.h>

#include "ml.h"


MLLines * ml_lines_new (void)
{
  MLLines *out;
  
  out = g_new (MLLines,1);
  out->lines = g_ptr_array_new ();
  out->tips = g_hash_table_new (g_direct_hash, g_direct_equal);
  
  return out;
}



void ml_lines_free (MLLines *lines)
{
  guint i;

  if (lines) {
    for (i = 0; i < lines->lines->len; i++)
      ml_line_free ((MLLine*)g_ptr_array_index (lines->lines, i));
    g_ptr_array_free (lines->lines, TRUE);

    g_hash_table_destroy (lines->tips);
    g_free (lines);
  }
}



void ml_lines_add_line (MLLines *lines, MLLine *line)
{
  g_assert (lines != NULL);
  g_assert (line != NULL);

  g_ptr_array_add (lines->lines, line);
  
  g_assert (!g_hash_table_lookup (lines->tips, GUINT_TO_POINTER(line->tip[0])));
  g_assert (!g_hash_table_lookup (lines->tips, GUINT_TO_POINTER(line->tip[1])));

  g_hash_table_insert (lines->tips, GUINT_TO_POINTER(line->tip[0]), line);
  g_hash_table_insert (lines->tips, GUINT_TO_POINTER(line->tip[1]), line);
}



void ml_lines_stick (MLLines *lines, 
    MLLine *line, MLLine *line2, MLLineAtom *glue,
    MLTLine type)
{
  guint8 tip, tip2;

  tip = (line->tip[1] == glue->tip[0] || line->tip[1] == glue->tip[1]) ? 1:0;
  g_assert (line->tip[tip] == glue->tip[0] || line->tip[tip] == glue->tip[1]); 
  
  tip2 = (line2->tip[1] == glue->tip[0] || line2->tip[1] == glue->tip[1]) ? 1:0;
  g_assert (line2->tip[tip2]==glue->tip[0] || line2->tip[tip2]==glue->tip[1]);
  
  line->type = type;
  
  g_assert (g_hash_table_lookup (lines->tips, GUINT_TO_POINTER (line->tip[0])));
  g_assert (g_hash_table_lookup (lines->tips, GUINT_TO_POINTER (line->tip[1])));
  g_hash_table_remove (lines->tips, GUINT_TO_POINTER (line->tip[0]));
  g_hash_table_remove (lines->tips, GUINT_TO_POINTER (line->tip[1]));

  if (line == line2) {
    ml_line_append (line, tip, line2, tip2, glue);
    
    /* smycka nema zadny konec */

    return;
  }
 
  g_assert (g_hash_table_lookup (lines->tips, GUINT_TO_POINTER(line2->tip[0])));
  g_assert (g_hash_table_lookup (lines->tips, GUINT_TO_POINTER(line2->tip[1])));
  g_hash_table_remove (lines->tips, GUINT_TO_POINTER (line2->tip[0]));
  g_hash_table_remove (lines->tips, GUINT_TO_POINTER (line2->tip[1]));
  
  ml_line_append (line, tip, line2, tip2, glue);
  
  g_hash_table_insert (lines->tips, GUINT_TO_POINTER (line->tip[0]), line);
  g_hash_table_insert (lines->tips, GUINT_TO_POINTER (line->tip[1]), line);
 
  g_ptr_array_remove (lines->lines, line2);

  ml_line_free (line2);
}
