/*
 *   MAPA - penalty of step
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: penalty.c,v 1.12 2001/04/11 12:04:00 mapa Exp $
 */

#include "ml.h"
#include "mlconfig.h"

#define CONNECT_QUALITY_MAX_SQRT 150
#define CONNECT_QUALITY_PERFECT 64

#define CONNECT_SORT_LINE 4
#define CONNECT_LOOP_MIN_LEN 25
#define CONNECT_STEPS_BACK 3


MLFFloat ml_penalty_step_cross (const MLStep *curr, MLSeparations pix, 
    guint8 step_direct, MLFFloat new_direct, MLFFloat line_direct,
    gboolean is_cross)
{
  return is_cross ? 
    curr->len + ml_directb_diff (curr->direct_in, step_direct) :
    ML_CONNECT_QUALITY_MAX;
}


MLFFloat ml_penalty_step_over (const MLStep *curr, MLSeparations pix, 
    guint8 step_direct, MLFFloat new_direct, MLFFloat line_direct,
    gboolean is_cross)
{
  MLFFloat penalty_length, penalty_direct;
  
  if (ml_directb_diff (curr->direct_in, step_direct) > 2)
    return ML_CONNECT_QUALITY_MAX;
  
  if (line_direct == ML_DIRECTF_UNKNOWN) 
    return ML_CONNECT_QUALITY_MAX;

  penalty_direct = (ml_directf_diff (line_direct,new_direct)*13) >> ML_FSHIFT;

  penalty_direct *= penalty_direct;

  penalty_length = ((step_direct%2) ? ML_FIXED_1*141/100 : ML_FIXED_1) / 45;

  if (pix & ML_LAYER_BK) 
    return penalty_direct / 2 + penalty_length / 6;
  else
    /* I must be worst the SPACE penalty */
    return penalty_direct + penalty_length + 1; 
}


MLFFloat ml_penalty_step (const MLStep *curr, MLSeparations pix, 
    guint8 step_direct, MLFFloat new_direct, MLFFloat line_direct,
    gboolean is_cross)
{
  MLFFloat penalty_length, penalty_direct;
 
  if (ml_directb_diff (curr->direct_in, step_direct) > 2)
    return ML_CONNECT_QUALITY_MAX;

  penalty_length = ((step_direct%2) ? ML_FIXED_1*141/100 : ML_FIXED_1) / 45;

  if (line_direct == ML_DIRECTF_UNKNOWN) 
    return penalty_length + penalty_length / 2;
  
  penalty_direct = (ml_directf_diff (line_direct,new_direct)*13) >> ML_FSHIFT;
  penalty_direct *= penalty_direct;


  return penalty_direct + penalty_length;
}



MLFFloat ml_penalty_connect (const MLStep *curr, MLSeparations pix, 
    guint8 step_direct, MLFFloat new_direct, MLFFloat line_direct,
    gboolean is_cross,
    const MLLineAtom *me, guint8 me_tip,
    const MLLineAtom *neighbour, guint8 neighbour_tip)
{
  MLFFloat dir;

  if (me == neighbour) 
    if (curr->len < ML_CONNECT_LOOP_MIN || me->len < ML_CONNECT_LOOP_MIN)
      return ML_CONNECT_QUALITY_MAX;
  
  dir = ml_lineAtom_direct (neighbour, neighbour_tip);
  dir = ml_directf_uturn (dir);
  
  return ml_directf_diff (dir, line_direct) / (ML_FIXED_1/8);	/* max = 0.5 */
}
