/*
 *   MAPA - basic tools for manipulation with a picture
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: picture.c,v 1.27 2001/03/29 06:07:26 mapa Exp $
 */

#include <unistd.h>
#include <stdio.h>
#include <png.h>

#include "ml.h"


MLPictureUI8* ml_pic_new (MLCoord rows, MLCoord cols)
{
  MLPictureUI8 *out;
  
  out = g_new (MLPictureUI8,1);
  out->rows = rows;
  out->cols = cols;
  out->p = g_new0 (guint8, ML_PIC_SIZE (out));
  
  return out;
}


/* return NULL if error */
MLPictureSep* ml_pic_read (int fd)
{
  MLPictureSep *pic;
  guint32 cols, rows, tmp;
  gchar *row_curr;
  gint i, need, count;

  if (read (fd, &tmp, 4) != 4) return NULL;
  if (read (fd, &cols, 4) != 4) return NULL;
  if (read (fd, &rows, 4) != 4) return NULL;

  pic = g_new (MLPictureSep, 1);
  pic->rows = rows + 2;
  pic->cols = cols + 2;
  pic->p = g_new0 (MLSeparations, ML_PIC_SIZE (pic));

#ifdef DEBUG
  g_message ("natm picture (%dx%d) ...", rows,cols);
#endif

  for (i = 0; i < rows; i++) {
    row_curr = (gchar*)&ML_PIC_PIXEL (pic,i+1,1);
    need = cols * sizeof(MLSeparations);

    do {
      count = read (fd, row_curr, need);
      if (count < 0) {
	g_free (pic->p);
	g_free (pic);
	return NULL;
      }

      row_curr += count;
      need -= count;
    } while (need);
  }

  for (i = 0; i < ML_PIC_SIZE(pic); i++)
    pic->p[i] = GUINT16_FROM_LE (pic->p[i]);

#ifdef DEBUG
  g_message (" ... nateno.");
#endif

  return pic;
}


/*
   
typedef gboolean	(*MLDrawFunc)	(MLPictureUI8	*pic,
					 MLCoord	 coord,
					 guint8		 tip,
					 gpointer	 data);
   
void ml_pic_draw_direct (MLPictureUI8 *pic, 
    MLCoord start, MLFFloat direct, MLDrawFunc func, gpointer data)
{
  MLCoord r,c;
  MLFFloat fr,fc,dd,dd2,dx,dy;
  guint8 tip;
 
  fr = ML_PIC_COORD_ROW(pic,start) * ML_FIXED_1;
  fc = ML_PIC_COORD_COL(pic,start) * ML_FIXED_1;

  dd = direct % ML_FIXED_1;
  dd2 = ML_FIXED_1 - dd;

  switch (direct >> ML_FSHIFT) {
    case 0: dx = -dd;
	    dy = -ML_FIXED_1;
	    break;
    case 1: dx = -ML_FIXED_1;
	    dy = -dd2;
	    break;
    case 2: dx = -ML_FIXED_1;
	    dy = dd;
	    break;
    case 3: dx = -dd2;
	    dy = ML_FIXED_1;
	    break;
    case 4: dx = dd;
	    dy = ML_FIXED_1;
	    break;
    case 5: dx = ML_FIXED_1;
	    dy = dd2;
	    break;
    case 6: dx = ML_FIXED_1;
	    dy = -dd;
	    break;
    case 7: dx = dd2;
	    dy = -ML_FIXED_1;
	    break;
    default: dx = dy = 0;
  }
 
  tip = 1;

  for (;;) {
    r = fr >> ML_FSHIFT;
    c = fc >> ML_FSHIFT;
    
    if (func (pic,ML_PIC_COORD(pic,r,c),tip,data) ||
	r == 0 || c == 0 || r == pic->rows-1 || c == pic->cols-1)
      return;
    
    tip = 0;
    
    fr += dx;
    fc += dy;
  }
  
  return;
}
*/


void ml_pic_draw_lineAtom (MLPictureUI8 *pic, 
    MLLineAtom *atom, guint8 tips, guint8 body)
{
  MLCoord coord;
  gint i;
  
  coord = atom->tip[0];
  pic->p[coord] = tips;
  
  for (i = 0; i < atom->len; i++) {
    switch (atom->data[i]) {
      case 0: coord = ML_PIC_COORD0(pic,coord); break;
      case 1: coord = ML_PIC_COORD1(pic,coord); break;
      case 2: coord = ML_PIC_COORD2(pic,coord); break;
      case 3: coord = ML_PIC_COORD3(pic,coord); break;
      case 4: coord = ML_PIC_COORD4(pic,coord); break;
      case 5: coord = ML_PIC_COORD5(pic,coord); break;
      case 6: coord = ML_PIC_COORD6(pic,coord); break;
      case 7: coord = ML_PIC_COORD7(pic,coord); break;
      default: g_assert_not_reached ();
    }
     
    pic->p[coord] = body; 
  }

  pic->p[coord] = tips;
  g_assert (atom->tip[1] == coord);
}


void ml_pic_draw_line (MLPictureUI8 *pic, MLLine *line, 
    guint8 tips, guint8 body)
{
  guint i;
  
  for (i = 0; i < line->atoms->len; i++)
    ml_pic_draw_lineAtom (pic, (MLLineAtom*)g_ptr_array_index (line->atoms,i),
	body, body);
  
  pic->p[line->tip[0]] = tips;
  pic->p[line->tip[1]] = tips;
}
