/*
 *   MAPA - napojeni linii
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: glue.c,v 1.1 2001/04/19 20:07:21 mapa Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "ml.h"
#include "mlt.h"


void G_GNUC_NORETURN fatal_error (char *msg, ...);

struct RGB {
  MLPictureUI8 *picR;
  MLPictureUI8 *picG;
  MLPictureUI8 *picB;
};

  

int main (int argc, char **argv)
{
  gint in;
  struct RGB rgb;
  MLPictureSep *pic;
  MLPictureUI8 *thin;
  MLGlueParam param;
  MLAtoms *atoms;
  MLGlue *glue;
  MLSeparations mask;
  gchar *fn;
  
  if (argc < 2 || argc > 3)
    fatal_error ("Usage: %s mask [pclfile]\nmask: one from\n"
	"  0 LighttYELLOW\n  1 DarkYELLOW\n  2 LightGREEN\n"
        "  3 MiddleGREEN\n  4 DarkGREEN\n  5 LightBLUE\n"
	"  6 DarkBLUE\n  7 LightBROWN\n  8 DarkBROWN\n  9 BLACK\n  10 ERR\n",
	argv[0]);

  if (argc == 3) {
    in = open (argv[2], O_RDONLY);

    if (in < 0)
      fatal_error ("%s: %s", argv[0], argv[2]);

  } else
    in = 0;

  mask = (MLSeparations)(1<<strtoul (argv[1], NULL, 0));

  pic = ml_pic_read (in);
  if (pic == NULL)
    fatal_error ("Read error");
  
  thin = ml_pic_thin (pic, mask);
  atoms = ml_pic_find_atoms (thin);
  ml_atoms_classify (atoms, pic, thin);
  
  /* pic_glue_default */
  
  rgb.picR = ml_pic_new (pic->rows, pic->cols);
  rgb.picG = ml_pic_new (pic->rows, pic->cols);
  rgb.picB = ml_pic_new (pic->rows, pic->cols);
 
#ifdef DEBUG
  g_message (" cross-glue ");
#endif

  param.step_penalty = ml_penalty_step_cross;
  param.connect_penalty = ml_penalty_connect;
  param.max_penalty = ML_FIXED_1 / 16;
  param.glue_type = MLTG_CROSS;
  param.debug = NULL;
  param.debug = rgb.picR;
    
  glue = ml_pic_glue (pic, thin, atoms, &param, NULL);


  if (!(mask & ML_LAYER_BK)) {
#ifdef DEBUG
    g_message (" overprint-glue ");
#endif
    param.step_penalty = ml_penalty_step_over;
    param.max_penalty = ML_FIXED_1 / 4;
    param.glue_type = MLTG_OVERPRINT;
    param.debug = rgb.picG;
    
    glue = ml_pic_glue (pic, thin, atoms, &param, glue);
  }
  
  
  /* space */
#ifdef DEBUG
  g_message (" space-glue ");
#endif
  param.step_penalty = ml_penalty_step;
  param.max_penalty = ML_FIXED_1;
  param.glue_type = MLTG_SPACE;
  param.debug = rgb.picB;
    
  glue = ml_pic_glue (pic, thin, atoms, &param, glue);
  
#ifdef DEBUG
  g_message ("|{glue}| = %d", glue->atoms->len);
#endif

  /* show area */
  fn = g_strdup_printf ("%s.png", argv[0]);
  if (ml_pic_write (fopen (fn, "wb"), rgb.picR, rgb.picG, rgb.picB) < 0) 
    fatal_error ("Write error (%s)", fn);
  g_free (fn);

  ml_pic_free (rgb.picR);
  ml_pic_free (rgb.picG);
  ml_pic_free (rgb.picB);
  ml_glue_free (glue);
  ml_atoms_free (atoms);
  ml_pic_free (thin);
  ml_pic_free (pic);
 
#ifdef DEBUG 
  g_blow_chunks ();
  g_mem_chunk_info ();
  g_mem_profile ();
#endif
  
  return 0;
}
