/*
 *   MAPA - find latoms
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: latoms.c,v 1.3 2001/03/23 13:41:46 mapa Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "ml.h"
#include "mlt.h"


void G_GNUC_NORETURN fatal_error (char *msg, ...);

struct RGB {
  MLPictureUI8 *picR;
  MLPictureUI8 *picG;
  MLPictureUI8 *picB;
};


int main (int argc, char **argv)
{
  gint in;
  struct RGB rgb;
  MLPictureSep *pic;
  MLPictureUI8 *thin;
  MLLineAtom *atom;
  MLAtoms *atoms;
  MLSeparations mask;
  gchar *fn;
  guint i;

  if (argc < 2 || argc > 3)
    fatal_error ("Usage: %s mask [pclfile]\nmask: one from\n"
	"  0 LighttYELLOW\n  1 DarkYELLOW\n  2 LightGREEN\n"
        "  3 MiddleGREEN\n  4 DarkGREEN\n  5 LightBLUE\n"
	"  6 DarkBLUE\n  7 LightBROWN\n  8 DarkBROWN\n  9 BLACK\n  10 ERR\n",
	argv[0]);

  if (argc == 3) {
    in = open (argv[2], O_RDONLY);

    if (in < 0) 
      fatal_error ("%s: %s", argv[0], argv[2]);

  } else 
    in = 0;

  mask = (MLSeparations)(1<<strtoul (argv[1], NULL, 0));
  
  pic = ml_pic_read (in);
  if (pic == NULL)
    fatal_error ("Read error");
  
  thin = ml_pic_thin (pic, mask);
  atoms = ml_pic_find_atoms (thin);
  ml_atoms_classify (atoms, pic, thin);
  
  /* show LONG */
  rgb.picR = ml_pic_new (pic->rows, pic->cols);
  rgb.picG = ml_pic_new (pic->rows, pic->cols);
  rgb.picB = ml_pic_new (pic->rows, pic->cols);
  
  for (i = 0; i < atoms->atoms->len; i++) {
    atom = (MLLineAtom*)g_ptr_array_index (atoms->atoms,i);

    switch (atom->type.line) {
      case MLTA_MARK:
      case MLTA_NOTLINE: ml_pic_draw_lineAtom (rgb.picB, atom, 255, 200); break;
      case MLTA_RASTER: ml_pic_draw_lineAtom (rgb.picR, atom, 255, 200); break;
      case MLTA_LINE: 
      case MLTA_UNKNOWN: 
			ml_pic_draw_lineAtom (rgb.picR, atom, 255, 200);
			ml_pic_draw_lineAtom (rgb.picG, atom, 255, 200);
			ml_pic_draw_lineAtom (rgb.picB, atom, 255, 200);
    }
  }

  fn = g_strdup_printf ("%s.png", argv[0]);
  if (ml_pic_write (fopen (fn, "wb"), rgb.picR, rgb.picG, rgb.picB) < 0) 
    fatal_error ("Write error (%s)", fn);
  g_free (fn);

  ml_pic_free (rgb.picR);
  ml_pic_free (rgb.picG);
  ml_pic_free (rgb.picB);

  ml_atoms_free (atoms);
  ml_pic_free (thin);
  ml_pic_free (pic);
  
 
#ifdef DEBUG 
  g_blow_chunks ();
  g_mem_chunk_info ();
  g_mem_profile ();
#endif
  
  return 0;
}
