/*
 *   MAPA - show lines
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: lines.c,v 1.5 2001/04/19 15:36:54 mapa Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "ml.h"
#include "mlt.h"


void G_GNUC_NORETURN fatal_error (char *msg, ...);

struct RGB {
  MLPictureUI8 *picR;
  MLPictureUI8 *picG;
  MLPictureUI8 *picB;
};


static void show_line (MLLine *line, struct RGB *rgb, MLTLine type)
{
  guint i;
  MLLineAtom *atom;
  guint8 max;
 
  if (line->type != type)
    return;
  
  max = (line->atoms->len < 2) ? 128 : 220;
  
  g_print ("*");
  
  for (i = 0; i < line->atoms->len; i++) {
    atom = (MLLineAtom*)g_ptr_array_index (line->atoms,i);
 
    if (atom->penalty == 0) { 
      /* atom */ 
      ml_pic_draw_lineAtom (rgb->picR, atom, 255, max);
      ml_pic_draw_lineAtom (rgb->picG, atom, 255, max);
      ml_pic_draw_lineAtom (rgb->picB, atom, 255, max);
    } else {
      switch (atom->type.glue) {
	case MLTG_CROSS: ml_pic_draw_lineAtom (rgb->picR, atom, 255, max); break;
	case MLTG_OVERPRINT: ml_pic_draw_lineAtom (rgb->picG, atom, 255,max); break;
	case MLTG_SPACE: ml_pic_draw_lineAtom (rgb->picB, atom, 255,max); break;
	default: g_print ("?");
      }
    }
  }
}


int main (int argc, char **argv)
{
  gint in;
  struct RGB rgb;
  MLPictureSep *pic;
  MLPictureUI8 *thin;
  MLAtoms *atoms;
  MLGlue *glue;
  MLLines *lines;
  MLSeparations mask;
  gchar *fn;
  guint i;

  if (argc < 2 || argc > 3)
    fatal_error ("Usage: %s mask [pclfile]\nmask: one from\n"
	"  0 LighttYELLOW\n  1 DarkYELLOW\n  2 LightGREEN\n"
        "  3 MiddleGREEN\n  4 DarkGREEN\n  5 LightBLUE\n"
	"  6 DarkBLUE\n  7 LightBROWN\n  8 DarkBROWN\n  9 BLACK\n  10 ERR\n",
	argv[0]);

  if (argc == 3) {
    in = open (argv[2], O_RDONLY);

    if (in < 0) 
      fatal_error ("%s: %s", argv[0], argv[2]);

  } else 
    in = 0;

  mask = (MLSeparations)(1<<strtoul (argv[1], NULL, 0));
  
  pic = ml_pic_read (in);
  if (pic == NULL)
    fatal_error ("Read error");
  
  thin = ml_pic_thin (pic, mask);
  atoms = ml_pic_find_atoms (thin);
  ml_atoms_classify (atoms, pic, thin);
  
  glue = ml_pic_glue_default (pic, thin, atoms, mask);
  
#ifdef DEBUG
  g_message ("|{glue}| = %d", glue->atoms->len);
#endif

  lines = ml_atoms_to_lines (atoms); 
  do {
    i = ml_pic_lines_connect (pic, lines, glue, NULL); 
#ifdef DEBUG
    g_message ("Spojeno: %u", i);
#endif
  } while (i);

  /* show LONG */
  rgb.picR = ml_pic_new (pic->rows, pic->cols);
  rgb.picG = ml_pic_new (pic->rows, pic->cols);
  rgb.picB = ml_pic_new (pic->rows, pic->cols);
  
  for (i = 0; i < lines->lines->len; i++) 
    show_line((MLLine*)g_ptr_array_index (lines->lines,i), &rgb, MLTL_LONG);
  
  fn = g_strdup_printf ("%s_long.png", argv[0]);
  if (ml_pic_write (fopen (fn, "wb"), rgb.picR, rgb.picG, rgb.picB) < 0) 
    fatal_error ("Write error (%s)", fn);
  g_free (fn);

  ml_pic_free (rgb.picR);
  ml_pic_free (rgb.picG);
  ml_pic_free (rgb.picB);
 

  /* show DASHED */
  rgb.picR = ml_pic_new (pic->rows, pic->cols);
  rgb.picG = ml_pic_new (pic->rows, pic->cols);
  rgb.picB = ml_pic_new (pic->rows, pic->cols);
  
  for (i = 0; i < lines->lines->len; i++) 
    show_line((MLLine*)g_ptr_array_index (lines->lines,i), &rgb, MLTL_DASHED);
  
  fn = g_strdup_printf ("%s_dash.png", argv[0]);
  if (ml_pic_write (fopen (fn, "wb"), rgb.picR, rgb.picG, rgb.picB) < 0) 
    fatal_error ("Write error (%s)", fn);
  g_free (fn);

  ml_pic_free (rgb.picR);
  ml_pic_free (rgb.picG);
  ml_pic_free (rgb.picB);
 

  /* show DOTTED */
  rgb.picR = ml_pic_new (pic->rows, pic->cols);
  rgb.picG = ml_pic_new (pic->rows, pic->cols);
  rgb.picB = ml_pic_new (pic->rows, pic->cols);
  
  for (i = 0; i < lines->lines->len; i++) 
    show_line((MLLine*)g_ptr_array_index (lines->lines,i), &rgb, MLTL_DOTTED);
  
  fn = g_strdup_printf ("%s_dot.png", argv[0]);
  if (ml_pic_write (fopen (fn, "wb"), rgb.picR, rgb.picG, rgb.picB) < 0) 
    fatal_error ("Write error (%s)", fn);
  g_free (fn);

  ml_pic_free (rgb.picR);
  ml_pic_free (rgb.picG);
  ml_pic_free (rgb.picB);
  

  ml_lines_free (lines);
  ml_glue_free (glue);
  ml_atoms_free (atoms);
  ml_pic_free (thin);
  ml_pic_free (pic);
  
 
#ifdef DEBUG 
  g_blow_chunks ();
  g_mem_chunk_info ();
  g_mem_profile ();
#endif
  
  return 0;
}
