/*
 *   MAPA - convertor PCL to PGMs
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: pcl2pgm.c,v 1.6 2000/12/05 18:17:38 mapa Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <pgm.h>
#include <glib.h>

#include "ml.h"


#ifdef DEBUG
#define OUT(layer) \
	g_message ("ukldm vrstvu %s:", #layer); \
	sprintf (file, "%s_%s.pgm", argv[1], #layer); \
	if ((out=fopen(file,"wb"))==NULL) g_error("Can't create %s", file); \
	write_layer_pgm (out,pic,ML_LAYER_##layer); \
	fclose (out);
#else
#define OUT(layer) \
	sprintf (file, "%s_%s.pgm", argv[1], #layer); \
	if ((out=fopen(file,"wb"))==NULL) g_error("Can't create %s", argv[1]); \
	write_layer_pgm (out,pic,ML_LAYER_##layer); \
	fclose (out);
#endif

void G_GNUC_NORETURN fatal_error (char *msg, ...);


static void write_layer_pgm (FILE *fp, MLPictureSep *pic, 
    MLSeparations layer_mask)
{
  register int i;
  register int j;
  gray *row;

  pgm_writepgminit (fp, pic->cols, pic->rows, 255, PGM_FORMAT);

  row = pgm_allocrow (pic->cols);

  for (i=0; i<pic->rows; i++) {
    for (j=0; j<pic->cols; j++) 
      row[j] = ((ML_PIC_PIXEL(pic,i,j) & layer_mask) > 0) ? 255 : 0;
    pgm_writepgmrow(fp, row, pic->cols, 255, PGM_FORMAT);
  }

  pgm_freerow(row);
}


int main (int argc, char **argv)
{
  int in;
  FILE *out;
  MLPictureSep *pic;
  char file[1024];
  
  pgm_init (&argc, argv);
  
  if (argc != 2)
    fatal_error ("Usage: %s <basefilename>", argv[0]);
 
  sprintf (file, "%s.pcl", argv[1]);
  if ((in=open (file, O_RDONLY)) < 0)
    fatal_error ("Can't open %s", file);
    
  if ((pic = ml_pic_read (in)) == NULL)
    fatal_error ("Can't read %s", argv[1]);
  
  close (in);
 
  OUT(LtYL);
  OUT(DkYL);
  OUT(LtGR);
  OUT(MdGR);
  OUT(DkGR);
  OUT(LtBL);
  OUT(DkBL);
  OUT(LtBR);
  OUT(DkBR);
  OUT(BK);
  OUT(ERR);
  
  ml_pic_free (pic);

  g_blow_chunks ();
  g_mem_chunk_info ();
  g_mem_profile ();

  return 0;
}
