/*
 *   MAPA - convertor PCL to PNG
 *
 *   (c) 2001 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: pcl2png.c,v 1.1 2001/03/23 12:49:13 mapa Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <glib.h>

#include "ml.h"
#include "mlt.h"

#define ML_RGB_LtYL   0xffe080
#define ML_RGB_DkYL   0xffc000
#define ML_RGB_LtGR   0xc0f0c0
#define ML_RGB_MdGR   0x00ff00
#define ML_RGB_DkGR   0x00c000
#define ML_RGB_LtBL   0x7b7bff
#define ML_RGB_DkBL   0x0000ff
#define ML_RGB_LtBR   0xbd9b7a
#define ML_RGB_DkBR   0xa52a2a
#define ML_RGB_BK     0x000000
#define ML_RGB_ERR    0xff0000



void G_GNUC_NORETURN fatal_error (char *msg, ...);

struct Layers {
  guint16 layer;
  guint32 rgb;
};

const struct Layers layers[] = {
  { ML_LAYER_LtYL, ML_RGB_LtYL },
  { ML_LAYER_DkYL, ML_RGB_DkYL },
  { ML_LAYER_LtGR, ML_RGB_LtGR },
  { ML_LAYER_MdGR, ML_RGB_MdGR },
  { ML_LAYER_DkGR, ML_RGB_DkGR },
  { ML_LAYER_LtBL, ML_RGB_LtBL },
  { ML_LAYER_DkBL, ML_RGB_DkBL },
  { ML_LAYER_LtBR, ML_RGB_LtBR },
  { ML_LAYER_DkBR, ML_RGB_DkBR },
  { ML_LAYER_BK  , ML_RGB_BK   },
  { ML_LAYER_ERR , ML_RGB_ERR  } 
};


int main (int argc, char **argv)
{
  int in;
  FILE *f;
  MLPictureSep *pic;
  MLPictureUI8 *outR, *outG, *outB;
  char file[1024];
  MLCoord coord;
  gint i;
  gint r,g,b;
  
  if (argc != 2)
    fatal_error ("Usage: %s <basefilename>", argv[0]);
 
  sprintf (file, "%s.pcl", argv[1]);
  if ((in=open (file, O_RDONLY)) < 0)
    fatal_error ("Can't open %s", file);
    
  if ((pic = ml_pic_read (in)) == NULL)
    fatal_error ("Can't read %s", argv[1]);
  
  close (in);
  
  outR = ml_pic_new (pic->rows, pic->cols);
  outG = ml_pic_new (pic->rows, pic->cols);
  outB = ml_pic_new (pic->rows, pic->cols);
  
  for (coord = 0; coord < ML_PIC_SIZE (pic); coord++) {
    r = g = b = 255;
    
    for (i = 0; i < sizeof (layers) / sizeof (struct Layers); i++)
      if (pic->p[coord] & layers[i].layer) {
	r = MAX (0, (gint)((layers[i].rgb>>16)&255) + r - 255);
	g = MAX (0, (gint)((layers[i].rgb>>8)&255) + g - 255);
	b = MAX (0, (gint)((layers[i].rgb)&255) + b - 255);
      }
    
    outR->p[coord] = r;
    outG->p[coord] = g;
    outB->p[coord] = b;
  }

  
  sprintf (file, "%s.png", argv[1]);
  if ((f = fopen (file, "wb")) == NULL)
    fatal_error ("Can't create %s", file);
  
  ml_pic_write (f, outR, outG, outB);
  
  ml_pic_free (outB);
  ml_pic_free (outG);
  ml_pic_free (outR);
  ml_pic_free (pic);

#ifdef DEBUG
  g_blow_chunks ();
  g_mem_chunk_info ();
  g_mem_profile ();
#endif

  return 0;
}
