/*
 *   MAPA - convertor PGMs to PCL
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: pgm2pcl.c,v 1.4 2000/12/15 08:00:02 mapa Exp $
 */

#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <pgm.h>

#include "ml.h"

#ifdef DEBUG
#define IN(layer)				G_STMT_START{		\
        g_message ("natm vrstvu %s:", #layer);			\
	sprintf (file, "%s_%s.pgm", argv[1], #layer);			\
	if ((in=fopen(file,"rb")) == NULL) 				\
		fatal_error ("Can't open %s",file);			\
	pic = read_layer (in, ML_LAYER_##layer);			\
	fclose (in);				}G_STMT_END
#else
#define IN(layer)				G_STMT_START{		\
	sprintf (file, "%s_%s.pgm", argv[1], #layer);			\
	if ((in=fopen(file,"rb")) == NULL) 				\
		fatal_error ("Can't open %s",file);			\
	pic = read_layer (in, ML_LAYER_##layer); 			\
	fclose (in);				}G_STMT_END
#endif

void G_GNUC_NORETURN fatal_error (char *msg, ...);

static MLPictureSep *read_layer (FILE *fp, MLSeparations layer_mask)
{
  static int cols = 0;
  static int rows = 0;
  register int i;
  register int j;
  static MLPictureSep *pic;
  int c,r,format;
  gray maxval;
  gray *row;

  pgm_readpgminit (fp, &c, &r, &maxval, &format);

  if (cols == 0 && rows == 0) {
    cols=c;
    rows=r;
    pic = g_new (MLPictureSep,1);
    pic->cols = c;
    pic->rows = r;
    pic->p = g_new0 (MLSeparations, ML_PIC_SIZE(pic));
  
  } else {
    if (cols != c || rows != r) fatal_error ("Sizes aren't same!");
  }
    
  row = pgm_allocrow (cols);
  
  for (i=0; i<r; i++) {
    pgm_readpgmrow(fp, row, c, maxval, format);
    for (j=0; j<c; j++) 
      ML_PIC_PIXEL(pic,i,j) |= (row[j]) ? layer_mask : 0;
  }
  
  pgm_freerow(row);
  
  return pic;
}



int main (int argc, char **argv)
{
  FILE *in;
  MLPictureSep *pic;
  char file[1024];
  int out;
  register int r;
  register int c;
  gint32 tmp;
  MLSeparations *row;
  
  pbm_init (&argc, argv);
  
  if (argc != 2)
    fatal_error ("Usage: %s <basefilename>", argv[0]);
 
  IN(LtYL);
  IN(DkYL);
  IN(LtGR);
  IN(MdGR);
  IN(DkGR);
  IN(LtBL);
  IN(DkBL);
  IN(LtBR);
  IN(DkBR);
  IN(BK);
  IN(ERR);
  
  sprintf (file, "%s.pcl", argv[1]);
  if ((out=creat(file, 0664)) < 0)
    fatal_error ("Can't creat %s", file);
  
  g_message ("ukldm picture ...");

  tmp = 2;
  g_return_val_if_fail (write (out, &tmp, 4) == 4, 1);

  tmp = pic->cols - 2;
  g_return_val_if_fail (write (out, &tmp, 4) == 4, 1);

  tmp = pic->rows - 2;
  g_return_val_if_fail (write (out, &tmp, 4) == 4, 1);
 
  row = g_new (MLSeparations, pic->cols-2);

  for (r = 1; r < pic->rows-1; r++) {
    for (c = 1; c <pic->cols-1; c++)
      row[c-1] = ML_PIC_PIXEL(pic,r,c);

    if (write (out, row, (pic->cols-2)*2) != (pic->cols-2)*2)
      fatal_error ("Can't write to %s", file);
  }
 
  g_free (row);
   
  ml_pic_free (pic);

  return 0;
}
