/*
 *   MAPA - make a PNG picture
 *
 *   (c) 2000 Jan Lana (jlan3028@ss1000.ms.mff.cuni.cz)
 *
 *   $Id: picwrite.c,v 1.2 2000/12/13 12:19:06 mapa Exp $
 */

//#include <unistd.h>
#include <png.h>

#include "mlt.h"

gint ml_pic_write (FILE *out, 
    MLPictureUI8 *picR, MLPictureUI8 *picG, MLPictureUI8 *picB)
{
  volatile MLCoord rows, cols;	/* longjmp need this 'volatile' */
  png_structp png_ptr;
  png_infop info_ptr;
  png_byte *row;
  gint i,j;
	 
  rows = cols = 0;

  /* test vstupu */
  if (picR != NULL) {
    rows = picR->rows;
    cols = picR->cols;
  }

  if (picG != NULL) {
    if (!rows) {
      rows = picG->rows;
      cols = picG->cols;
    } else
      g_assert (rows == picG->rows && cols == picG->cols);
  }

  if (picB != NULL) {
    if (!rows) {
      rows = picB->rows;
      cols = picB->cols;
    } else
      g_assert (rows == picB->rows && cols == picB->cols);
  }
  
  g_assert (cols > 0 && rows > 0);

  /* ulozeni obrazku */
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
      NULL, NULL, NULL);
  g_assert (png_ptr != NULL);
  
  info_ptr = png_create_info_struct(png_ptr);
  g_assert (info_ptr != NULL);
  
  if (setjmp(png_ptr->jmpbuf)) {
    fclose(out);
    png_destroy_write_struct (&png_ptr, (png_infopp)NULL);
    return -1;
  }

  png_init_io(png_ptr, out);
  png_set_IHDR(png_ptr, info_ptr, cols-2, rows-2, 8, 
      PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
      PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
  png_set_compression_level(png_ptr, Z_BEST_SPEED);

  png_write_info(png_ptr, info_ptr);
  png_set_packing(png_ptr);
  
  row = g_new0 (png_byte, 3*(cols-2));

#ifdef DEBUG
  g_message ("ukldm picture ...");
#endif
  

  for (i = 1; i < (rows-1); i++) {
    for (j = 1; j < (cols-1); j++) {
      row[(j-1)*3] = (picR==NULL) ? 0 : ML_PIC_PIXEL(picR,i,j);
      row[(j-1)*3+1] = (picG==NULL) ? 0 : ML_PIC_PIXEL(picG,i,j);
      row[(j-1)*3+2] = (picB==NULL) ? 0 : ML_PIC_PIXEL(picB,i,j);
    }

    png_write_row(png_ptr, row);
  }

#ifdef DEBUG
  g_message (" ... uloeno.");
#endif

  png_write_end(png_ptr, info_ptr);
  png_destroy_write_struct(&png_ptr, &info_ptr);
  
  g_free (row);

  fclose (out);

  return 0;
}
